/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

/**
 * Total number of completions to offer
 */
#define MAX_ALTERNATIVES 50

/**
 * Avaible completions
 */
GCompletion *URLCompletion = NULL;

/** 
 * List of posible completions for the current entry
 */
GList *posible_completions = NULL;

/**
 * The alternatives window
 */ 
GtkWidget *alternatives_window = NULL;

/* Private functions */
void on_auto_completion_button_clicked (GtkButton *button, gpointer user_data);

/**
 * Adds an url to the list of completions
 */
void
auto_completion_add_url (gchar *url)
{
	if (URLCompletion) {
		GList *list = g_list_append (NULL, url);
		g_completion_add_items (URLCompletion, list);
		g_list_free (list);	
	} else {
		URLCompletion = g_completion_new (NULL);
		auto_completion_add_url (url);
	}
}

/**
 * Destroys the alternatives window and reset the list of _current_ posible 
 * completions. Does not clear the whole completions list.
 */
void 
auto_completion_reset (void)
{
	posible_completions = NULL;
	if (alternatives_window != NULL)
		gtk_widget_destroy (alternatives_window);
	alternatives_window = NULL;
}

/** 
 * Tries to complete an url 
 * Builds a list of posible completions in posible_completions (don't try to
 * free this!) and retruns the common prefix (if any) or NULL if there are no
 * completions. Remember to free the returned string!
 */
gchar *
auto_completion_complete_url (gchar *prefix)
{
	gchar *common;
	if (URLCompletion == NULL) return NULL;
	posible_completions = g_completion_complete(URLCompletion, prefix, &common);
	return common;
}

/** 
 * Like auto_completion_complete_url but tries to prepend "http://" and similar
 */
gchar *
auto_completion_complete_url_extended (gchar *prefix)
{
	gchar *common = NULL;
	gchar *tries[] = {"", "http://", "http://www.", "file://", NULL};
	int i;
	if (URLCompletion == NULL) return NULL;
	for (i = 0; (common == NULL) && (tries[i] != NULL); i++) {
		gchar *extended_prefix = g_strconcat (tries[i], prefix, NULL);
		posible_completions = g_completion_complete(URLCompletion, 
							    extended_prefix, &common);
		g_free(extended_prefix);
	}
	return common;
}

/**
 * Displays the list of posible completions under the given widget 
 */
/* TODO: Sort the list, take out duplicates */
void 
auto_completion_display_alternatives (GtkWidget *w)
{
	gint x, y, height, width, depth;
	GtkWidget *table = gtk_vbox_new (TRUE, 0);
	GList *l;
	int count = 0;
	if (alternatives_window)
		gtk_widget_destroy (alternatives_window);
	alternatives_window = gtk_window_new (GTK_WINDOW_POPUP);
	gtk_container_add (GTK_CONTAINER(alternatives_window), table);
	for (l =  posible_completions; (l != NULL) && (count < MAX_ALTERNATIVES);
	     l = g_list_next(l), count++) {
		GtkWidget *b = gtk_button_new_with_label (l->data);
		gtk_container_add (GTK_CONTAINER(table), b);
		gtk_signal_connect (GTK_OBJECT(b), "clicked", 
				    GTK_SIGNAL_FUNC(on_auto_completion_button_clicked), 
				    w);
	}	
	gdk_window_get_geometry (w->window, &x, &y, &width, &height, &depth);
	gdk_window_get_deskrelative_origin (w->window, &x, &y);
	y += height;
	gtk_widget_set_uposition (alternatives_window, x, y);
	gtk_widget_show_all (alternatives_window);
}

void
on_auto_completion_button_clicked (GtkButton *button, gpointer user_data)
{
	if (GTK_IS_ENTRY (user_data)) {
		gchar *text = GTK_LABEL (GTK_BIN (button)->child)->label;
		gtk_entry_set_text (GTK_ENTRY (user_data), text);
	}
}

/** 
 * Adds the history of a GnomeEntry to the list of completions 
 */
/* Actually, I think that this may be unnecessary. */
void 
auto_completion_add_from_entry (GnomeEntry *e) 
{
	static gboolean done = FALSE;
	if (!done) { /* do this only once */
		struct item {
			gboolean save;
			gchar *text;
		};
		GList *list;
		for (list = e->items; list != NULL; list = list->next) {
			struct item *i = list->data;
			auto_completion_add_url (i->text);
		}	
	}
}
