/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"
#include "libgnomeui/gnome-window-icon.h"

/* local function prototypes */
static void browser_init_data (GaleonBrowser *gb);

/* the global list of all GaleonBrowser windows */
GSList *window_list = NULL;

/* gtkmozembed type of drag and drop */
static const GtkTargetEntry drop_types[] = {
	{ "GALEON_URL", 0, DND_TARGET_GALEON_URL },
	{ "_NETSCAPE_URL", 0, DND_TARGET_NETSCAPE_URL },
	{ "STRING", 0, DND_TARGET_STRING }
};
static const gint drop_types_num_items = (sizeof (drop_types) / sizeof (GtkTargetEntry));

/* url type of drag and drop */
static const GtkTargetEntry url_drag_types[] = {
	{ "GALEON_BOOKMARK", 0, DND_TARGET_GALEON_BOOKMARK},
	{ "GALEON_URL", 0, DND_TARGET_GALEON_URL },
	{ "_NETSCAPE_URL", 0, DND_TARGET_NETSCAPE_URL },
	{ "STRING", 0, DND_TARGET_STRING }
};
static const gint url_drag_types_num_items = (sizeof (url_drag_types) 
					      / sizeof (GtkTargetEntry));

/**
 * browser_create_default: create a new browser pointing at the page
 * specified by configuration
 */
GaleonBrowser *
browser_create_default (gboolean first_window)
{
	GaleonBrowser *browser;
	gchar *home_page_url;
	gchar *last_page_url;
	gint page_type;

        /* get location of home page and last page */
	last_page_url = history_get_last_url();
	home_page_url = gnome_config_get_string(CONF_GENERAL_HOMEPAGE);
	
	/* find out where we're supposed to start */
	if (first_window)
		page_type = gnome_config_get_int(CONF_GENERAL_HOMEPAGE_TYPE);
	else
		page_type = gnome_config_get_int(CONF_GENERAL_NEWPAGE_TYPE);

	/* create a browser */
	browser = browser_create();
	
	/* go to the appropriate page */
	if (page_type == HOME_PAGE && home_page_url != NULL)
	{
		browser_load_url (browser, home_page_url);
	}
	else if (page_type == LAST_PAGE && last_page_url != NULL)
	{
		browser_load_url (browser, last_page_url);
	}
	else if (page_type == BLANK_PAGE)
	{
		/* do nothing, we already have a blank page loaded */
	}
	else if (page_type == 3)
	{
		/* FIXME: remove this after a version or so -- MattA */

		if (first_window)
			gnome_config_set_int(CONF_GENERAL_FIRSTPAGE_TYPE,
					     HOME_PAGE);
		else
			gnome_config_set_int(CONF_GENERAL_NEWPAGE_TYPE,
					     HOME_PAGE);

		gnome_config_set_string(CONF_GENERAL_HOMEPAGE, MYPORTAL_URL);

		/* do the right thing this time */
		browser_create_default(first_window);
	}
	else
	{
		g_error("unknown page type");
	}
	
	/* free allocated strings */
	if (home_page_url) g_free(home_page_url);
	if (last_page_url) g_free(last_page_url);
	
	return browser;
}

/**
 * browser_load_url: interpret and load a URL into a given browser
 */
void
browser_load_url (GaleonBrowser *browser, const gchar *url)
{
	/* check arguments */
	g_assert(browser != NULL);
	g_assert(url != NULL);

	/* error if we can't handle this URL */
	if (handle_foreign_protocols(url))
		return;

	/* load the URL */
	if (strcmp(url, MYPORTAL_URL) == 0)
	{
		/* generate "My Portal" from bookmarks */
		portal_render_into_browser(browser);
	}
	else
	{
		/* load the url */
		gtk_moz_embed_load_url (browser->mozEmbed, url);
	}
}

/**
 * browser_create_from_url: create a browser from a given url string
 */
GaleonBrowser *
browser_create_from_url (const gchar *url)
{
	GaleonBrowser *browser;

	/* check argument */
	g_assert(url != NULL);

	/* error if we can't handle this URL */
	if (handle_foreign_protocols(url))
		return NULL;

	/* create a window */
	browser = browser_create();

	/* load the url */
	browser_load_url(browser, url);
	
	return browser;
}

/**
 * browser_create_from_url_view_source: create a browser from a given 
 * url string  in view source mode
 */
GaleonBrowser *
browser_create_from_url_view_source (const gchar *url)
{
	GaleonBrowser *browser;
	GtkWidget *view_source_mode_menuitem;

	/* check argument */
	g_assert(url != NULL);

	/* error if we can't handle this URL */
	if (handle_foreign_protocols(url))
		return NULL;

	/* create a window */
	browser = browser_create();

	/* set view source mode */
	view_source_mode_menuitem = 
		glade_lookup_widget(browser->WMain, "view_source_mode");
	gtk_check_menu_item_set_active 
		(GTK_CHECK_MENU_ITEM (view_source_mode_menuitem), TRUE);
	
	/* load the url */
	browser_load_url(browser, url);

	return browser;
}

/* signals to connect on each browser widget */
static const struct
{ 
	char *event; 
	void *func; /* should be a GtkSignalFunc or similar */
}
signal_connections[] =
{
	{ "location",        location_changed_cb  },
	{ "title",           title_changed_cb     },
	{ "net_start",       load_started_cb      },
	{ "net_stop",        load_finished_cb     },
	{ "net_state",       net_status_change_cb },
	{ "progress",        progress_change_cb   },
	{ "link_message",    link_message_cb      },
	{ "js_status",       js_status_cb         },
	{ "open_uri",        open_uri_cb          },
	{ "visibility",      visibility_cb        },
	{ "destroy_browser", destroy_brsr_cb      },	
#ifdef M18
	{ "dom_mouse_click", dom_mouse_click_cb },
	{ "dom_mouse_down", dom_mouse_down_cb },
	{ "dom_mouse_up", dom_mouse_up_cb },
	{ "dom_mouse_over", dom_mouse_over_cb },
	{ "dom_key_press", dom_key_press_cb },
	{ "size_to", size_to_cb },
#endif
	/* terminator -- must be last in the list! */
	{ NULL, NULL } 
};

/**
 * browser_create: create a main window widget with a browser widget
 * inside it.
 */
GaleonBrowser *
browser_create (void)
{
	static gboolean preferences_set = FALSE;
	gchar *blank_page = "<HTML><BODY></BODY></HTML>";
        GaleonBrowser *gb;
        GtkWidget *window;
	GladeXML *gxml;
	GnomeEntry *ge;
	gint width, height;
	gint i;

	/* use libglade to build the window */
	gxml = glade_xml_new (glade_file (), "WMain");
	window = glade_xml_get_widget (gxml, "WMain");

	/* set mini icon */
	gnome_window_icon_set_from_file (GTK_WINDOW(window), gnome_pixmap_file ("galeon.png"));

	/* find the default width and height */
	width = gnome_config_get_int (CONF_APPEARANCE_WINWIDTH);
	height = gnome_config_get_int (CONF_APPEARANCE_WINHEIGHT);

	/* setup the window */
	gtk_window_set_default_size (GTK_WINDOW(window), width, height);
    
	/* don't know why libglade does not set this */
	GNOME_APP(window)->menubar = glade_xml_get_widget (gxml, "menubar1"); 

	/* allocate and initialise the GaleonBrowser structure */
	gb = g_new0 (GaleonBrowser, 1);
	gb->WMain = window;
	gb->mozEmbed = (GtkMozEmbed *) gtk_moz_embed_new ();
	gtk_moz_embed_set_chrome_mask(gb->mozEmbed, GTK_MOZ_EMBED_FLAG_ALLCHROME);
	gtk_signal_connect(GTK_OBJECT(gb->mozEmbed), "destroy",
			   GTK_SIGNAL_FUNC(on_browser_destroy), gb);
  	browser_init_data (gb);

	/* connect the signals */
	glade_xml_signal_autoconnect_full (gxml, (GladeXMLConnectFunc) glade_signal_connect_func, gb);

 	/* reference widget and add data hooks */
	gtk_widget_ref (GTK_WIDGET(gb->mozEmbed));
	gtk_object_set_data_full (GTK_OBJECT(gb->mozEmbed), "mozEmbed",
				  GTK_WIDGET(gb-> mozEmbed),
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_object_set_data (GTK_OBJECT(window), "mozilla", gb);
	gnome_app_set_contents (GNOME_APP(window), GTK_WIDGET(gb->mozEmbed));

	/* add it to the list of browser windows */
	window_list = g_slist_append(window_list, gb);

	/* connect signals */
	for (i = 0; signal_connections[i].event != NULL; i++)
	{
		gtk_signal_connect_while_alive (GTK_OBJECT(gb->mozEmbed),
						signal_connections[i].event,
						signal_connections[i].func, gb, GTK_OBJECT(gb->mozEmbed));
	}

	/* connect open window signal according to the new windows pref */
	if (gnome_config_get_int(CONF_GENERAL_NEWWINDOWS)==0)
	{
		gtk_signal_connect_while_alive (GTK_OBJECT(gb->mozEmbed),
						"new_window",
						new_window_cb, gb, GTK_OBJECT(gb->mozEmbed));
	}


	/* create the bookmarks menu */
	bookmarks_create_menu (gb);

	/* setup the toolbar style for this window */
	set_toolbar_style_window (gb);

	/* setup the history dropdown menu */
	ge = GNOME_ENTRY(gb->eUrl);
	gnome_entry_set_history_id (ge, "url");
	gnome_entry_load_history (ge);
	auto_completion_add_from_entry (GNOME_ENTRY(ge));

	/* set gtkmozembed drag and drop destination */
	gtk_drag_dest_set (GTK_WIDGET(gb->mozEmbed), GTK_DEST_DEFAULT_ALL,
			   drop_types, drop_types_num_items,
			   GDK_ACTION_COPY | GDK_ACTION_MOVE |
			   GDK_ACTION_LINK | GDK_ACTION_ASK );

	/* set url entry drop destination */
	gtk_drag_dest_set 
		(GTK_WIDGET (gnome_entry_gtk_entry 
			     (GNOME_ENTRY (gb->main_toolbar_gnomeentry))),
		 GTK_DEST_DEFAULT_ALL,
		 drop_types, drop_types_num_items,
		 GDK_ACTION_COPY);
	gtk_drag_dest_set 
		(GTK_WIDGET (gnome_entry_gtk_entry 
			     (GNOME_ENTRY (gb->location_toolbar_gnomeentry))),
		 GTK_DEST_DEFAULT_ALL,
		 drop_types, drop_types_num_items,
		 GDK_ACTION_COPY);

	/* set url drag source */
	gtk_drag_source_set (GTK_WIDGET (gb->location_label1->parent->parent),
			     GDK_BUTTON1_MASK, url_drag_types, url_drag_types_num_items,
			     GDK_ACTION_COPY|GDK_ACTION_LINK);
	gtk_drag_source_set (GTK_WIDGET (gb->location_label2->parent->parent),
			     GDK_BUTTON1_MASK, url_drag_types, url_drag_types_num_items,
			     GDK_ACTION_COPY|GDK_ACTION_LINK);

	/* set bookmark menuitem drop destination */
	gtk_drag_dest_set (glade_xml_get_widget (gxml, "bookmarks"), 
			   GTK_DEST_DEFAULT_ALL,
			   url_drag_types, url_drag_types_num_items,
			   GDK_ACTION_COPY);

	/* set gtmozembed drag and drop signals */
	gtk_signal_connect(GTK_OBJECT(gb->mozEmbed), "drag_drop",
			   GTK_SIGNAL_FUNC(drag_drop), gb);
	gtk_signal_connect(GTK_OBJECT(gb->mozEmbed), "drag_data_received",
			   GTK_SIGNAL_FUNC(drag_data_received), gb);

	/* set preferences */
	if (!preferences_set) 
	{
		mozilla_prefs_set ();
		preferences_set = TRUE;
	}

	/* wait for the window to be fully drawn */
	while (gtk_events_pending())
		gtk_main_iteration();

	/* render a blank page into the browser: means it's
 	 * always displaying something and we don't have the
	 * unpleasent sight of an see-through frame -- MattA */
	/* 2/9/2000: we now render blankness but use the about:blank
	 * URL. Otherwise, when loading of a page failed we'd still
	 * get the empty frame. -- MattA */
	gtk_moz_embed_render_data(gb->mozEmbed,
				  blank_page, strlen(blank_page),
				  "about:blank", "text/html");

	/* show the main window */
	gtk_widget_show (window);

	/* return the completed GaleonBrowser */
	return gb;
}

/**
 * browser_show_error: show a browser error message
 */
void browser_show_error(GaleonBrowser *browser, char *errormsg)
{
	/* check preferences about where to display the error message */
	int error = gnome_config_get_int ("/galeon/Appearance/error=0");
	
	if (error == SHOW_ON_STATUS)
	{ 
		/* show the error message on status bar */
		browser->statusMessage = errormsg;
		browser_update_status_bar_text (browser);
	}
	else if (error == SHOW_ON_DIALOG) 
	{
		/* show an error dialog */
		gnome_error_dialog (errormsg);
	}
}

/**
 * browser_progress_clear: clear all infos about download progress
 */
void
browser_progress_clear(GaleonBrowser *browser)
{
	/* set all progress values to 0 */ 
	browser->loadPercent = 0;
	browser->bytesLoaded = 0;
	browser->maxBytesLoaded = 0;
}

/**
 * browser_update_status_bar_test: update the status bar of the toplevel browser window
 */
void
browser_update_status_bar_text(GaleonBrowser *browser)
{
	gchar message[256];	

	gnome_appbar_pop(GNOME_APPBAR(browser->appbar1));

	if (browser->tempMessage) 
	{
		/* fill the status bar text with the current temporary message */
		gnome_appbar_push(GNOME_APPBAR(browser->appbar1),browser->tempMessage);
	}
	else if (browser->statusMessage)
	{

		if ((browser->loadPercent) 
		    && (browser->bytesLoaded <= browser->maxBytesLoaded))
		{
			/* fill the status bar text with progress info: percentual and kb */
			g_snprintf(message, 255, _("%s (%d%% complete, %d Kb of %d Kb loaded)"), 
				   browser->statusMessage, browser->loadPercent, 
				   (!browser->bytesLoaded) ? 0 : browser->bytesLoaded/1024,
				   (!browser->maxBytesLoaded) ? 0 : browser->maxBytesLoaded/1024);
		}
		else if (browser->bytesLoaded)
		{
			/* fill the status bar text with progress info: only kb */
			g_snprintf(message, 255, _("%s (%d Kb loaded)"), browser->statusMessage, 
				   (!browser->bytesLoaded) ? 0 : browser->bytesLoaded/1024);
		}
		else
		{
			g_snprintf(message, 255, "%s", browser->statusMessage);
		}


		gnome_appbar_push(GNOME_APPBAR(browser->appbar1),message);
	}

}

/**
 * browser_update_temp_message: update the temporary message
 */
void
browser_update_temp_message(GaleonBrowser *browser, const char *message)
{
	/* free the previous message */
	if (browser->tempMessage) 
	{
		g_free(browser->tempMessage);
	}
	
	/* fill the new temporary message */
	if (message) 
	{
		browser->tempMessage = g_strdup(message);
	}	
	else
	{	
		browser->tempMessage = NULL;
	}
	browser_update_status_bar_text(browser);
}

/**
 * browser_update_nav_buttons: update back and forward toolbar buttons status
 */
void
browser_update_nav_buttons (GaleonBrowser *browser)
{
	gboolean can_go_back, can_go_forward;

	/* query mozilla */
	can_go_back = gtk_moz_embed_can_go_back (browser->mozEmbed);
	can_go_forward = gtk_moz_embed_can_go_forward (browser->mozEmbed);

	/* set widget status appropriately */
	gtk_widget_set_sensitive (browser->BBack, can_go_back);
	gtk_widget_set_sensitive (browser->BForward, can_go_forward);
}

/**
 * browser_progress_action: update the status bar progress
 */
gboolean
browser_progress_action (GaleonBrowser *browser)
{
	/* FIXME comment here */
	GtkWidget *progress;
	gfloat val;

	g_return_val_if_fail(browser->appbar1 != NULL, FALSE);
	progress = GTK_WIDGET(gnome_appbar_get_progress(GNOME_APPBAR(browser->appbar1)));

	/* update progressive loader "activity" indicator */
	gtk_progress_set_activity_mode(GTK_PROGRESS(progress), TRUE);
	val = gtk_progress_get_value(GTK_PROGRESS(progress));
	if (++val > 100)
		val = 0;
	gtk_progress_set_value(GTK_PROGRESS(progress), val);

	/* returns true (i.e. call again) if this browser is still active */
	return browser->active;
}

/**
 * browser_init_data: init GaleonBrowser data structure
 */
static void
browser_init_data (GaleonBrowser *gb)
{
	gb->spinner = NULL;
	gb->statusMessage = NULL;
	gb->link = NULL;
	gb->loadPercent = 0;
	gb->bytesLoaded = 0;
	gb->maxBytesLoaded = 0;
	gb->tempMessage = NULL;
	gb->active = FALSE;
	gb->cmenu_timer = NULL;
	gb->cmenu_timeout_id = -1;
	gb->bookmarks_toolbars = NULL;
	gb->bookmarks_tooltips = NULL;

	/* fill GUI components */
	gb->BBack = glade_lookup_widget (gb->WMain, "BBack");
	gb->BForward = glade_lookup_widget (gb->WMain, "BForward");
	gb->BRefresh = glade_lookup_widget (gb->WMain, "BRefresh");
	gb->zoom_spin = GTK_EDITABLE (glade_lookup_widget (gb->WMain, "zoom_spin"));
	gb->BStart = glade_lookup_widget (gb->WMain, "BStart");
	gb->BStop = glade_lookup_widget (gb->WMain, "BStop");
	gb->appbar1 = glade_lookup_widget (gb->WMain, "appbar1");
	gb->main_toolbar_gnomeentry = glade_lookup_widget (gb->WMain, "main_toolbar_gnomeentry");	
	gb->location_toolbar_gnomeentry = glade_lookup_widget (gb->WMain, "location_toolbar_gnomeentry");	
	gb->main_toolbar = glade_lookup_widget (gb->WMain, "main_toolbar");
	gb->go_toolbar1 = glade_lookup_widget (gb->WMain, "go_toolbar1");	
	gb->go_toolbar2 = glade_lookup_widget (gb->WMain, "go_toolbar2");	
	gb->location_dockitem = glade_lookup_widget (gb->WMain, "location_dockitem");	
	gb->main_dockitem = glade_lookup_widget (gb->WMain, "main_dockitem");
	gb->main_toolbar_hbox = glade_lookup_widget (gb->WMain, "main_toolbar_hbox");
	gb->location_label1 = glade_lookup_widget (gb->WMain, "location_label1");
	gb->location_label2 = glade_lookup_widget (gb->WMain, "location_label2");
	gb->menubar = glade_lookup_widget (gb->WMain, "menubar1");	
	gb->view_source_mode = glade_lookup_widget (gb->WMain, "view_source_mode");	
	gb->view_toolbar = glade_lookup_widget (gb->WMain, "view_toolbar");	
	gb->view_menubar = glade_lookup_widget (gb->WMain, "view_menubar");	
	gb->view_statusbar = glade_lookup_widget (gb->WMain, "view_statusbar");	
	gb->view_fullscreen = glade_lookup_widget (gb->WMain, "view_fullscreen");	
	gb->drag_pixmap = glade_lookup_widget (gb->WMain, "drag_pixmap");
}

/**
 * browser_show_find_dialog: show the find dialog
 */
void browser_show_find_dialog (GaleonBrowser *browser, gpointer docshell)
{
	GtkWidget *dFind;
	GladeXML *gxml;

	gxml = glade_xml_new(glade_file(), "dFind");
	glade_xml_signal_autoconnect_full (gxml, (GladeXMLConnectFunc) glade_signal_connect_func, browser);
	dFind = glade_xml_get_widget(gxml, "dFind");

	gtk_object_set_data (GTK_OBJECT(dFind), "docshell", docshell);
	
	gtk_widget_show(dFind);

	browser->start_find = TRUE;
}

/**
 * browser_show_open_dialog: show the open dialog
 */
void browser_show_open_dialog(GaleonBrowser *browser)
{
	GtkWidget *fs;
	GladeXML *gxml;

	gxml = glade_xml_new(glade_file(), "fsopen");
	glade_xml_signal_autoconnect_full (gxml, (GladeXMLConnectFunc) glade_signal_connect_func, browser);
	fs = glade_xml_get_widget(gxml, "fsopen");

	gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),gnome_config_get_string(CONF_DIR_OPEN));

	gtk_widget_show(fs);
}

/**
 * browser_show_openurl_dialog: show the open url dialog
 */
void browser_show_openurl_dialog (GaleonBrowser *browser)
{
	GtkWidget *dialog = NULL;

	dialog = gnome_request_dialog(
		FALSE, "Open Url", NULL, 100, (GnomeStringCallback) on_open_url_ok_cb, browser, NULL
		);

	gnome_dialog_run(GNOME_DIALOG (dialog));
}

/**
 * browser_edit_bookmarks: show the edit bookmarks dialog
 */
void browser_edit_bookmarks (GaleonBrowser *browser)
{
	g_return_if_fail (bookmarks_editor != NULL);
	g_return_if_fail (GTK_IS_WIDGET (bookmarks_editor->dialog));
	gtk_widget_show (bookmarks_editor->dialog);
	gdk_window_raise (bookmarks_editor->dialog->window);
}

/**
 * browser_show_temporary_bookmarks: show the temporary bookmarks dialog
 */
void browser_show_temporary_bookmarks (GaleonBrowser *browser)
{
	if (!temp_bookmarks_window)
		temp_bookmarks_init ();

	gtk_widget_show (temp_bookmarks_window->dialog);
	gdk_window_raise (temp_bookmarks_window->dialog->window);

	temp_bookmarks_window->browser = browser;
}

/**
 * browser_show_history: show the history dialog
 */
void browser_show_history (GaleonBrowser *browser)
{
	g_return_if_fail (dHistory != NULL);

	/* this is to know wich window last invoked the history dialog */
	gtk_object_set_data(GTK_OBJECT(dHistory), "WINDOW", (gpointer) browser->WMain);
	gtk_widget_show(dHistory);
	gdk_window_raise(dHistory->window);
}

/**
 * browser_add_temporary_bookmark: add a temporary bookmark
 */
void browser_add_temporary_bookmark (GaleonBrowser *browser)
{
	gchar *name = g_strdup (gtk_moz_embed_get_title (browser->mozEmbed));
	gchar *url = gtk_editable_get_chars (GTK_EDITABLE (browser->entry1), 0, -1);
	add_temp_bookmark (SITE, name, url, NULL);
	g_free (url);
	g_free (name);
}

/**
 * browser_show_save_dialog: show the save dialog
 */
void browser_show_save_dialog (GaleonBrowser *browser, gpointer docshell)
{
	GtkWidget *fs;
	GladeXML *gxml;

	gxml = glade_xml_new(glade_file(), "fssave");
	glade_xml_signal_autoconnect_full (gxml,(GladeXMLConnectFunc) glade_signal_connect_func, browser);
	fs = glade_xml_get_widget(gxml, "fssave");

	gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),gnome_config_get_string(CONF_DIR_SAVE));

	gtk_object_set_data (GTK_OBJECT(fs), "docshell", docshell);
	
	gtk_widget_show(fs);
}

/**
 * browser_toolbar_show: show the toolbar
 */
void browser_toolbar_show (GaleonBrowser *browser)
{
	gtk_widget_show (browser->main_dockitem);	

	if (gnome_config_get_int ("/galeon/Appearance/url_location=1")==0)
		gtk_widget_show (browser->location_dockitem);
	bookmarks_toolbar_set_visible (browser, TRUE);

}

/**
 * browser_toolbar_hide: hide the toolbar
 */
void browser_toolbar_hide (GaleonBrowser *browser)
{
	gtk_widget_hide (browser->main_dockitem);
	gtk_widget_hide (browser->location_dockitem);
	bookmarks_toolbar_set_visible (browser, FALSE);
		
	gtk_widget_queue_resize(GTK_WIDGET(browser->mozEmbed));
}

/**
 * browser_menubar_show: show the menubar
 */
void browser_menubar_show (GaleonBrowser *browser)
{
	gtk_widget_show (browser->menubar->parent);	
}

/**
 * browser_menubar_hide: hide the menubar
 */
void browser_menubar_hide (GaleonBrowser *browser)
{
	gtk_widget_hide (browser->menubar->parent);
	gtk_widget_queue_resize(GTK_WIDGET(browser->mozEmbed));
}

/**
 * browser_statusbar_show: show the statusbar
 */
void browser_statusbar_show (GaleonBrowser *browser)
{
	gtk_widget_show(browser->appbar1->parent);
}

/**
 * browser_statusbar_hide: hide the statusbar
 */
void browser_statusbar_hide (GaleonBrowser *browser)
{
	gtk_widget_hide(browser->appbar1->parent);
	gtk_widget_queue_resize(GTK_WIDGET(browser->mozEmbed));
}

void browser_view_source (GaleonBrowser *browser, gpointer docshell)
{
	if (gnome_config_get_bool (CONF_HANDLERS_USE_EXTERNAL_SOURCE_VIEWER)) {
		browser_view_source_external (browser, docshell);
	} 
	else 
	{
		gchar *url = NULL; 
		
		if (docshell)
		{
			url = mozilla_get_docshell_url (docshell);
		}
		else 
		{
			url = gtk_moz_embed_get_location (browser->mozEmbed);
		}

		if (url != NULL) 
		{
			browser_create_from_url_view_source (url);
		} 
		else 
		{
			gnome_error_dialog (
				_("Can't get the url for current page.\n"
				  "No source avaible."));
		}
	}
}

void 
browser_view_source_external (GaleonBrowser *browser, gpointer docshell)
{
	char *filename;
	pid_t pid;

	/* get a name for the temporary file */
	filename = tmpnam_ex (".html");
		
	if (!filename) {
		gnome_error_dialog ( _("Could not create a temporary file"));
		return;
	}
		
	/* save the file from mozilla to a temporary file */
	
	if (!mozilla_save (browser->mozEmbed, filename, docshell)) {
		gnome_error_dialog (_("Something went wrong while saving the "
				      "file to a temporary location"));
		g_free (filename);
		return;
	}
		
		
	/* Fork and invoke the external program. */
	pid = fork ();
	if (pid == 0) {
		/* child process */
		gchar *prog = gnome_config_get_string
			(CONF_HANDLERS_EXTERNAL_SOURCE_VIEWER);
		execlp (prog, prog, filename, NULL);
		g_warning (_("Error executing external program"));
		_exit (0);
	} else if (pid < 0) {
		/* error */
		g_error ("Error creating subprocess");
		return;
	} else {
		/* parent process */
		/* fork again and give the external program time to start  
		   before removing the temp file */
		pid_t pid2 = fork ();
		if (pid2 == 0) {
				/* child */
			sleep (45); /* 45 secs should be enough */
			remove (filename);
			_exit (0);
		} else if (pid2 < 0) {
			g_error ("Error creating subprocess");
		}
		g_free (filename);
	}
}

/**
 * browser_close: close a browser
 */
void browser_close (GaleonBrowser *browser)
{
	gtk_widget_destroy(browser->WMain);
}

/**
 * Open the frame pointed by the docshell 
 */
void 
browser_open_frame (GaleonBrowser *browser, gpointer docshell, gboolean same_window)
{
	gchar *url = NULL; 
	
	if (docshell) {
		url = mozilla_get_docshell_url (docshell);
	} else { /* should not reach here never... */
		url = gtk_moz_embed_get_location (browser->mozEmbed);
	}

	if (url != NULL) {
		if (same_window) {
			gtk_moz_embed_load_url (browser->mozEmbed, url);
		} else {
			browser_create_from_url (url);
		}
	}
}
