/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

/* local function prototypes */
static gint mime_compare(const MimeItem *m1, const MimeItem *m2);

/* the global list of MimeItems */
GList *mime_types = NULL;

/**
 * mime_add: add a mime type to the global list
 */
void
mime_add(gchar *name, gchar *program, gchar *extensions, MimeAction action)
{
	MimeItem *m;

	/* allocate and fill most of the structure */
	m = (MimeItem *) g_malloc (sizeof(MimeItem));
	m->name = g_strdup (name);
	m->program = g_strdup (program ? program : "");
	m->extensions = g_strsplit(extensions, ",", 0);
	m->action = action;

	/* add it to the global list of mime types in sort order */
	mime_types = g_list_insert_sorted(mime_types, m,
					  (GCompareFunc)mime_compare);
}

/**
 * mime_delete: remove a MimeItem from the global list
 */
void
mime_delete(MimeItem *m)
{
	/* free allocated strings */
	g_free(m->name);
	g_free(m->program);
	g_strfreev(m->extensions);

	/* good habit */
	memset(m, 0, sizeof(MimeItem));

	/* remove from the global list */
	g_list_remove(mime_types, m);
	
	/* free the item itself */
	g_free(m);
}

/**
 * mime_update: update the entries of a mime type
 */
void
mime_update(MimeItem *m, gchar *name, gchar *program, 
	    gchar *extensions, MimeAction action)
{
	/* free the old data */
	g_free(m->name);
	g_free(m->program);
	g_strfreev(m->extensions);
	
	/* insert the new data */
	m->name = g_strdup (name);
	m->program = g_strdup (program ? program : "");
	m->extensions = g_strsplit(extensions, ",", 0);
	m->action = action;
}

/**
 * mime_compare: find an ordering for MimeItems based on their names
 */
static gint
mime_compare(const MimeItem *m1, const MimeItem *m2)
{
	return (strcmp(m1->name, m2->name));
}


/**
 * get_mime_entry
 */
void
mime_get_entry(gchar **name, gchar **program, gchar **extensions,
	       MimeAction *action)
{
	GtkWidget *name_entry = glade_lookup_widget(GTK_WIDGET(prefs), "name_entry");
	GtkWidget *ext_entry = glade_lookup_widget(GTK_WIDGET(prefs), "ext_entry");
	GtkWidget *program_entry = glade_lookup_widget(GTK_WIDGET(prefs), "program_entry");
	GtkWidget *action_optionmenu = glade_lookup_widget(GTK_WIDGET(prefs), "action_optionmenu");

	*name = gtk_editable_get_chars(GTK_EDITABLE(name_entry), 0, -1);
	*program = gtk_editable_get_chars(GTK_EDITABLE(program_entry), 0, -1);
	*extensions = gtk_editable_get_chars(GTK_EDITABLE(ext_entry), 0, -1);
	*action = g_list_index(GTK_MENU_SHELL(GTK_OPTION_MENU(action_optionmenu)->menu)->children, gtk_menu_get_active(GTK_MENU(GTK_OPTION_MENU(action_optionmenu)->menu)));
}
