/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"
#include <dirent.h>

/* list of known protocols */
const char *known_protocols[] =
{
	"http",
	"https",
	"file",
	"about",
	"resource",
	"javascript",
	"jar",
       	NULL /* terminator: must be last! */
};

/* list of known mime types */
const char *known_mime_types[] =
{
	/* standard documents */
	"text/html",
	"text/plain",
	"text/x-perl", /* slashdot login! */

	/* standard images */
	"image/jpeg",
	"image/gif",
	"image/x-png",
	"image/png",

	/* protocols -- these are meta-mimetypes for when the real
	 * type has not yet been divined, e.g. http://some.site.com/ */
	"x-url/http",
	"x-url/ftp",

	/* meta-mimetypes of generated documents */
	"application/x-cgi",
	"application/x-asp",
	"application/x-php",
	"application/x-cgi",

	/* a misidentified type seen on the BBC website */
	"audio/x-stm", 

	/* terminator, must be last! */
	NULL
};

/* Local prototypes */
void add_bf_menu (GtkWidget *menu, char* label, int index, GaleonBrowser *browser);
void free_string_array (char *array[], int size);
static void unrecognised_protocol_cb(gint reply, gpointer url);

/**
 * glade_lookup_widget
 */
GtkWidget *
glade_lookup_widget (GtkWidget *widget, const gchar *widget_name)
{
	GtkWidget *found_widget, *parent;
	GladeXML *xml = NULL;

	while (xml == NULL)
	{
		/* the following line is to allow to override the
		 * GladeXML object or to set it to an object which was
		 * not created by libglade. Useful for popup menus */
		xml = gtk_object_get_data (GTK_OBJECT(widget), "widget_tree");
		if (!xml) xml = glade_get_widget_tree(widget);

		if (GTK_IS_MENU(widget))
			parent = gtk_menu_get_attach_widget (GTK_MENU(widget));
		else
			parent = widget->parent;

		if (parent == NULL) 
			break;

		widget = parent;
	}

	found_widget = glade_xml_get_widget(xml, widget_name);

	if (!found_widget)
		g_warning ("Widget not found: %s", widget_name);
 
	return found_widget;
}

/**
 * user_file: returns the pathname of galeon shared files (e.g., galeon.glade)
 *
 * fname: just the filename with no path information
 * critical: critical file? (halt if not found)
 */
gchar *
user_file(gchar *fname, gint critical)
{
	int i;
	static gchar *file = NULL;
	gchar * alternative[4];

	/* check to see if we've already located the file */
	/* and also that it's for the same file needed */
	if (file != NULL && !strcmp(file+strlen(file)-strlen(fname), fname))
		return file;

	/* try the default */
	file = g_strconcat (g_get_home_dir (), "/.galeon/", fname, NULL);
	
	/* success? */
	if (access (file, R_OK) == 0)
		return file;

	g_free(file);

	/* specify alternate locations in order of precedence */
	alternative[0] = fname;
	alternative[1] = g_strconcat("../", fname, NULL);
	alternative[2] = g_strconcat(SHARE_DIR "/", fname, NULL);
	alternative[3] = NULL;  /* NULL terminator needed */
	
	/* select one of the alternatives */
	for (i = 0; alternative[i] != NULL; i++)
	{
		file = alternative[i];
		if (access (file, R_OK) == 0)
		{
                        /* don't warn for the install default */
			if (i != 2)
				g_warning("Using %s (usually OK)\n", file);

			return file;
		}
	}

	/* if nothing then theres an error */
	if (critical)
		g_error(_("%s not found"), fname);
	else
		file = NULL;

	return NULL;
}

/* shortcut for old code */
inline gchar * glade_file()
{
	return user_file("galeon.glade", TRUE);
}

/*
 * glade_signal_connect_func: used by glade_xml_signal_autoconnect_full
 */
void
glade_signal_connect_func(const gchar *cb_name, GtkObject *obj, 
	const gchar *signal_name, const gchar *signal_data, GtkObject *conn_obj,
	gboolean conn_after, gpointer user_data)
{
	gpointer handler_func;
	
	/*g_print( "glade_signal_connect_func: cb_name = '%s', signal_name = '%s', signal_data = '%s'\n",
	  cb_name, signal_name, signal_data ); */
	
	if( g_module_symbol(mod_self, cb_name, &handler_func ) ) {
		/* found callback */
		if( conn_obj ) {
			if( conn_after ) {
				gtk_signal_connect_object_after( obj, signal_name, 
					handler_func, conn_obj );
			} else {
				gtk_signal_connect_object( obj, signal_name, 
					handler_func, conn_obj );
			}
		} else {
			/* no conn_obj; use standard connect */
			gpointer data = NULL;
			
			data = user_data;
			
			if( conn_after ) {
				gtk_signal_connect_after( obj, signal_name, 
					handler_func, data );
			} else {
				gtk_signal_connect( obj, signal_name, 
					handler_func, data );
			}
		}
	}
	else g_warning("callback function not found: %s", cb_name);

}

/**
 * handle_foreign_protocols: work out the mime type of a URL,
 * return TRUE if we handle it separately and FALSE if we want
 * mozilla to have a go
 */
gboolean
handle_foreign_mime_types (const char *url)
{
	const gchar *mime_type;
	gint i;

	/* guess the mime type from the url */
	/* DON'T free this -- it's (perhaps stupidly) a const string
	 * that's passed back by the gnome-mime infrastructure */
	mime_type = gnome_mime_type (url);

	/* if we found one check if it's one we know */
	if (mime_type != NULL)
	{
		/* if it's one we know we let mozilla display it */
		for (i = 0; known_mime_types[i] != NULL; i++)
		{
			if (strcmp(mime_type, known_mime_types[i]) == 0)
				return FALSE;
		}
	}
	else
	{
		/* perhaps it's something mozilla knows about... */
		return FALSE;
	}

/* autodownload--FIXME */

	/* what we *should* do here is let the user choose between
	 * opening, editing, running (etc) the mime type according
	 * to the gnome settings. However, we don't have any way of
	 * downloading-then-executing something: we'll need the
	 * gtm interface to be extended first. What we can do instead
	 * is to download it and save to disk */

	save_url(url);

	/* we've handled it */
	return TRUE;
}

/**
 * handle_foreign_protocols: work out what protocol is specified
 * by a URL, return TRUE if we find a handler and execute it,
 * otherwise return FALSE to tell mozilla to try to load it
 */
gboolean
handle_foreign_protocols (const char *url)
{
	gint i, length;
	gchar *protocol;
	gchar *key, *result;
	gchar *url_copy;

	/* find the colon */
	length = strlen(url);
	for (i = 0; i < length && url[i] != ':'; i++)
		;
	
	/* did we run over? */
	if (i == length)
	{
		/* don't know what to do here so leave it to the lizard */
		return FALSE;
	}

	/* okay, duplicate this */
	protocol = g_strndup(url, i);

        /* see if it's definitely a known protocol */
        for (i = 0; known_protocols[i] != NULL; i++)
        {
                if (strcasecmp (protocol, known_protocols[i]) == 0)
		{
			/* fine, give it to mozilla */
			g_free(protocol);
                        return FALSE;
		}
        }

	/* if it's FTP, check if the user has setup to use mozilla */
	if (strcasecmp (protocol, "ftp") == 0 &&
	    gnome_config_get_int ("/galeon/Handlers/ftp=0") == 0)
	{
			/* give it to mozilla */
			g_free(protocol);
                        return FALSE;
	}

	/* we don't know the protocol, check to see what GNOME has */
	/* unfortunately there's not an easy function to do this... */

	/* build the config key */
	key = g_strconcat("/Gnome/URL Handlers/", protocol, "-show", NULL);
	g_free(protocol);

	/* find it */
	result = gnome_config_get_string(key);
	g_free(key);
	if (result != NULL && strlen(result) != 0)
	{
		/* we have a GNOME handler for this, pass it through
		 * and tell mozilla not to try showing it */
		g_free(result);
		gnome_url_show(url);
		return TRUE;
	}

	/* free */
	if (result)
		g_free(result);

	/* no luck, so offer the user the option of trying the
	 * default handler -- we don't do this automatically in
	 * case the default handler is erroneously set to galeon */
	result = gnome_config_get_string("/Gnome/URL Handlers/default-show");

	/* check there is a default */
	if (result == NULL || strlen(result) == 0)
	{
		/* throw the error */
		gnome_error_dialog(_("Galeon cannot handle this protocol,\n"
				     "and no GNOME default handler is set"));

		/* free */
		if (result) g_free(result);

		/* don't let mozilla try blindly */
		return TRUE;
	}
 
	/* free */
	g_free(result);

	/* offer the choice */
	url_copy = g_strdup(url);
	gnome_question_dialog(_("The protocol specified is not recognised.\n\n"
				"Would you like to try the GNOME default?"),
			      unrecognised_protocol_cb, (gpointer)url_copy);

	/* keep mozilla out of it */
	return TRUE;
}

/**
 * first_time_cb: called if the user decides to import existing
 * netscape bookmarks or not
 */
static void
unrecognised_protocol_cb(gint reply, gpointer url)
{
        /* use default GNOME URL handler if asked to */
        if (reply == GNOME_YES)
	{
		gnome_url_show((gchar *)url);
	}

	/* free the copied url */
	g_free(url);
}

/**
 * save_url
 */
void
save_url (const gchar* url)
{
	if (gnome_config_get_bool("/galeon/Downloading/"
				  "ask_for_download_dir=true"))
	{
		ask_dir (url);
	}
	else
	{
		save_url_with_gtm (url);
	}
}

/**
 * save_url_with_gtm
 */
void
save_url_with_gtm (const gchar* url)
{  
	gboolean disable_auto_download;
	gchar *dir;

	/* get the download directory */
	dir = gnome_config_get_string ("/galeon/Downloading/download_dir");

	if (gnome_config_get_bool ("/galeon/Downloading/auto_download=true"))
		disable_auto_download = FALSE;
	else
		disable_auto_download = TRUE;

	gtm_add_url (g_strdup (url), dir, FALSE, disable_auto_download);		
		
	g_free(dir);
}

/**
 * ask_dir
 */
void
ask_dir (const gchar *url)
{
	GtkWidget *dirselect;
	GtkWidget *file_list;
	GladeXML *gxml;
	char *olddir;
	
	/* build the directory selector */
	gxml = glade_xml_new (glade_file(), "dirselection2");
	glade_xml_signal_autoconnect (gxml);
	dirselect = glade_xml_get_widget (gxml, "dirselection2");

	/* set it up with the standard download directory */
	olddir = gnome_config_get_string ("/galeon/Downloading/download_dir");
	gtk_file_selection_set_filename (GTK_FILE_SELECTION(dirselect),
					 olddir);
	gtk_object_set_data(GTK_OBJECT(dirselect), "url", g_strdup (url));

	/* don't allow the selection of files */
	file_list = GTK_FILE_SELECTION(dirselect)->file_list;
	gtk_widget_set_sensitive(file_list, FALSE);

	/* show it */
	gtk_widget_show(GTK_WIDGET(dirselect));
}


/*
 * g_str_is_url:
 * 
 * test a string for a loadable URL -- very general, not intended to be picky
 * 
 * returns 1 if string is a URL, otherwise returns 0
 */
int g_str_is_url(gchar * string)
{
	gchar * s;
       
	s = strstr(string, "://");

	if (s) {
		/* if there's characters before and after :// */
		if (strlen(s) > 3 /* length of "://" */  && s != string)
			return 1;
	}

	s = strstr(string, "www.");
	if (s) {
		/* if theres characters after and nothing before */
		if (strlen(s) > 4 /* length of "www." */ && s == string)
			return 1;
	}

	return 0;
}


/* file_exists:
 *
 * check if a local file exists, returns 1 if it does, 0 otherwise
 */
int file_exists(char * fname)
{
	FILE *f;
	
	if ((f = fopen(fname, "r"))) {
		fclose(f);
		return 1;
	}
	return 0;
}

gchar *
init_cache_dir(void)
{ 
	gchar *confdir = g_strconcat(g_get_home_dir(),"/.galeon",NULL);
	gchar *cachedir = g_strconcat(confdir,"/cache",NULL);

	if (access (cachedir, F_OK) == -1)
	{
		if (mkdir (cachedir, 488) != 0)
			g_error("Could not make directory (%s)", cachedir);
	}

	g_free(confdir);
	return cachedir;
}

/**
 * Frees an array of strings
 */ 
void 
free_string_array (char *array[], int size)
{
	int i;
	for (i = 0; i < size; i++)
		g_free (array[i]);
	g_free (array);
}

/**
 * Adds a menuitem to a back/forward history menu
 */
void
add_bf_menu (GtkWidget *menu, char* label, int index, GaleonBrowser *browser)
{
	GtkWidget *item = gtk_menu_item_new_with_label (label);
	gtk_widget_show (item);
	gtk_object_set_user_data (GTK_OBJECT (item), browser);
	gtk_menu_append (GTK_MENU (menu), item);
	gtk_signal_connect (GTK_OBJECT (item), "activate",
			    menuitem_bf_activated, GINT_TO_POINTER (index));
}

/** 
 * Creates the back history menu 
 */
GtkMenu *
create_back_menu (GaleonBrowser *b)
{
	int index, count, i;
	char **titles;
	GtkWidget *menu = gtk_menu_new ();

	if (!mozilla_sesion_history (b->mozEmbed, &titles, &count, &index))
	{
		return NULL;
	}

	for (i = index - 1; i >= 0; i--) 
		add_bf_menu (menu, titles[i], i, b);
	
	free_string_array (titles, count);
	return GTK_MENU(menu);
}

/**
 * Creates the forward history menu
 */
GtkMenu *
create_forward_menu (GaleonBrowser *b)
{
	int index, count, i;
	char **titles;
	GtkWidget *menu = gtk_menu_new ();

	if (!mozilla_sesion_history (b->mozEmbed, &titles, &count, &index))
	{
		return NULL;
	}	

	for (i = index + 1; i < count; i++)
		add_bf_menu (menu, titles[i], i, b);
	
	free_string_array (titles, count);
	return GTK_MENU(menu);
}

void
menu_position_under_widget (GtkMenu *menu, gint *x, gint *y, 
			    gpointer user_data)
{
	GtkWidget *w = GTK_WIDGET(user_data);
	gint px, py, width, height, depth;
	gdk_window_get_geometry(w->window, &px, &py, &width, &height, &depth);
	gdk_window_get_deskrelative_origin(w->window, &px, &py);
	*y = py + height;
	*x = px;
}

/**
 * Returns a temp filename wich ends in a given suffix and does not exist
 */
char *tmpnam_ex (char *suffix)
{
	gchar *s;
	gchar *t = NULL;
	gboolean found = FALSE;
	int tries = 0;
	while ((!found) && (tries++ < 100)) {
		FILE *f;
		s = g_malloc(L_tmpnam);
		tmpnam (s);
		if (t) g_free(t);
		t = g_strconcat (s, suffix, NULL);
		f = fopen (t, "r");
		if (f == NULL) {
			found = TRUE;
		} else {
			fclose (f);
			g_free (t);
			t = NULL;
		}
		g_free (s);
	}
	return t;
}

/**
 * copy_to_clipboard: Copies some text into the clipboard
 **/
void
copy_to_clipboard (const gchar *text)
{
	/* used to past text in the clipboard */
	static GtkWidget *clipboard_entry = NULL;
	/* FIXME: I think there's some api to do this (g[td]k_selection */
	
	if (!clipboard_entry) {
		clipboard_entry = gtk_entry_new();
		//gtk_widget_set_parent(clipboard_entry, browser->WMain);
		gtk_widget_ref (clipboard_entry);
	}
			
	g_return_if_fail (text != NULL);

	gtk_entry_set_text (GTK_ENTRY (clipboard_entry), text);
	gtk_editable_select_region (GTK_EDITABLE (clipboard_entry), 0, -1);
	gtk_editable_copy_clipboard (GTK_EDITABLE (clipboard_entry));
} 

void galeon_exit (void)
{
	/*FIXME all browsers should be closed */

	gnome_config_set_int("/galeon/General/crashed", FALSE);
	/* FIXME: need to sync config? */
	preferences_save();
	bookmarks_save();
	temp_bookmarks_save ();
	history_save();
	
	gtk_main_quit();
}

/*
 * remove_directory: remove a directory. CREDITS gnome-core and _vicious_ that gave me that suggestion
 */

void
remove_directory(const char *dirname, gboolean just_clean)
{
        DIR *dir;
        struct dirent *dent;
        char *oldcwd;

        dir = opendir (dirname);
        if(!dir) return;
        oldcwd = g_get_current_dir();

        chdir(dirname);
        while((dent = readdir (dir)) != NULL) {
                if(strcmp(dent->d_name,".")==0 ||
                   strcmp(dent->d_name,"..")==0)
                        continue;
                if(g_file_test(dent->d_name,G_FILE_TEST_ISDIR))
                        remove_directory(dent->d_name, FALSE);
                else
                        unlink(dent->d_name);
        }
        closedir(dir);
        chdir(oldcwd);

        if(!just_clean)
                rmdir(dirname);
        g_free(oldcwd);
}
