
/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

/* hack for dragging links */
static gchar *dragging_link = NULL;

/* type of drag and drop */
static const GtkTargetEntry link_drag_types[] = {
	{ "GALEON_URL", 0, DND_TARGET_GALEON_URL },
	{ "_NETSCAPE_URL", 0, DND_TARGET_NETSCAPE_URL },
	{ "STRING", 0, DND_TARGET_STRING }
};
static const gint link_drag_types_num_items = (sizeof (link_drag_types) 
					       / sizeof (GtkTargetEntry));
					       	       
static GtkTargetList *link_drag_types_tl = NULL;

/* CMENU_HACK stuff is common to CONTEXT_TIMER_MENU */
#ifdef CMENU_HACK
#define CONTEXT_TIMER_MENU
#endif


void link_drag_data_get (GtkWidget *widget, GdkDragContext *context,
			 GtkSelectionData *selection_data, 
			 guint info, guint time, 
			 GaleonBrowser *browser);


/**
 * new_window_cb: GTKMOZEMBED SIGNAL, emitted any time a new window is requested by the document 
 */
void 
new_window_cb (GtkMozEmbed *embed, GtkMozEmbed **retval, guint chromemask,GaleonBrowser *browser) 
{
	GaleonBrowser *gb;

#ifdef DEBUG_SIGNALS 
	g_print("new_window_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);

	/* create a new browser */
	gb = browser_create ();

	/* return the new browser to gtkmozembed */
	*retval = GTK_MOZ_EMBED (gb->mozEmbed);
}

/**
 * visibility_cb: GTKMOZEMBED SIGNAL, emitted when the toplevel window need to be showed or hidden
 */
void 
visibility_cb (GtkMozEmbed *embed, gboolean visibility, GaleonBrowser *browser) 
{
#ifdef DEBUG_SIGNALS
	g_print("visibility_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);

	/* show or hide browser toplevel window */
	if (visibility) 
		gtk_widget_show (browser->WMain);
	else 
		gtk_widget_hide (browser->WMain);
}

/**
 * destroy_brsr_cb: GTKMOZEMBED SIGNAL, emitted when the document has requested that the toplevel window be closed
 */
void 
destroy_brsr_cb (GtkMozEmbed *embed, GaleonBrowser *browser) 
{
#ifdef DEBUG_SIGNALS
	g_print("destroy_brsr_cb\n");
#endif

	g_return_if_fail(browser->WMain!=NULL);

	/* destroy the browser toplevel window */
	gtk_widget_destroy (browser->WMain);
}

/**
 * location_changed_cb: GTKMOZEMBED SIGNAL, emitted any time that the location of the document has changed
 */
void
location_changed_cb (GtkMozEmbed *embed, GaleonBrowser *browser)
{
	char *newLocation;
	int   newPosition = 0;

#ifdef DEBUG_SIGNALS
 	g_print("location_changed_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);

	newLocation = g_strdup (gtk_moz_embed_get_location (embed));

	if (newLocation)
	{
		/* change the url entry text */
		gtk_editable_delete_text (browser->entry1, 0, -1);
		if (strcmp(newLocation, "about:blank"))
			gtk_editable_insert_text (browser->entry1, newLocation, 
						  strlen (newLocation), &newPosition);
		g_free (newLocation);
	}
	else 
	{
		browser_update_temp_message (browser, 0);
	}
	
	browser_update_nav_buttons (browser);

	/* autosave the updated session */
	session_autosave();
}

/**
 * title_changed_cb: GTKMOZEMBED SIGNAL, emitted any time that the title of the document has changed
 */
void
title_changed_cb (GtkMozEmbed *embed,GaleonBrowser *browser)
{
	char *title;
	char *full_title;
	char *title_string = gnome_config_get_string (CONF_APPEARANCE_WINDOWS_TITLE);

#ifdef DEBUG_SIGNALS	
	g_print("title_changed_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);

	/* get the new document title */
	title = gtk_moz_embed_get_title(embed);

	/* if the document has no title set it to Untitled */
	if (!title || (strlen (title) == 0)) 
	{ 
		title = _("Untitled");
	}
	full_title = g_strdup_printf (title_string, title);

	/* set the toplevel window title to the document title */
	gtk_window_set_title (GTK_WINDOW(browser->WMain), full_title);
	if (full_title) g_free (full_title);
	if (title_string) g_free (title_string);
  
}

/**
 * load_started_cb: GTKMOZEMBED SIGNAL, emitted any time that the load of a document has been started
 */
void
load_started_cb (GtkMozEmbed *embed, GaleonBrowser *browser)
{
	GtkWidget *progress;

#ifdef DEBUG_SIGNALS
	g_print("load_started_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);
		
	progress = GTK_WIDGET(gnome_appbar_get_progress (GNOME_APPBAR(browser->appbar1)));

	/* update stop and refresh toolbar buttons */
	gtk_widget_set_sensitive (browser->BStop, TRUE);
	gtk_widget_set_sensitive (browser->BRefresh, FALSE);

	/* start the spinnet animation */
	if (browser->spinner)
		gnome_animator_start (GNOME_ANIMATOR(browser->spinner));

	/* initialize loading progress */
	browser_progress_clear (browser);

	/* indicate this browser has started loading */
	browser->active = TRUE;

	/* show the progess bar */
	gtk_widget_show(GTK_WIDGET(progress));

	g_timeout_add (100, (GSourceFunc) browser_progress_action, browser);

	browser_update_status_bar_text (browser);
}

/**
 * load_finished_cb: GTKMOZEMBED SIGNAL, emitted any time that the load of a document has finished
 */
void
load_finished_cb (GtkMozEmbed *embed, GaleonBrowser *browser)
{
	gchar *title, *url;
	history_item *hi;
	GtkWidget *progress;

#ifdef DEBUG_SIGNALS
	g_print("load_finished_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);

	progress = GTK_WIDGET (gnome_appbar_get_progress 
			       (GNOME_APPBAR(browser->appbar1)));

	/* disable the progress bar */
	gtk_progress_set_activity_mode (GTK_PROGRESS(progress), FALSE);
	gtk_widget_hide (GTK_WIDGET(progress));

	/* update stop and refresh toolbar buttons */
	gtk_widget_set_sensitive (browser->BStop, FALSE);
	gtk_widget_set_sensitive (browser->BRefresh, TRUE);

	/* stop the spinner animator */
	if(browser->spinner) 
	{
		gnome_animator_stop (GNOME_ANIMATOR(browser->spinner));
		gnome_animator_goto_frame (GNOME_ANIMATOR(browser->spinner), 0);
	}

	/* indicate that the browser is now stopped */
	/* FIXME: this is used to stop the 100ms timeout callback --
	 * can't we stop it in a more direct way?? -- MattA */
	browser->active = FALSE;

	/* clear progress infos */
	browser_progress_clear (browser);
	browser_update_status_bar_text (browser);
	gnome_appbar_set_progress (GNOME_APPBAR(browser->appbar1), 0);

	/* add the url to the history */
	title = g_strdup (gtk_moz_embed_get_title (browser->mozEmbed));
	url = g_strdup (gtk_moz_embed_get_location (browser->mozEmbed));
	hi = history_visited (url, title);

	/* add url to recent menu */
	if (hi) {
		history_add_recent_menu (hi, GNOME_APP(browser->WMain));
	}
}

/**
 * net_status_change_cb: GTKMOZEMBED SIGNAL, emitted any time that there is a change in the status of network loading
 */
void net_status_change_cb (GtkMozEmbed *embed, gint flags, guint status, GaleonBrowser *browser) 
{
#ifdef DEBUG_SIGNALS
	g_print("net_status_change_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);

	if (flags & GTK_MOZ_EMBED_FLAG_IS_REQUEST) {
		if (flags & GTK_MOZ_EMBED_FLAG_REDIRECTING)
			browser->statusMessage = _("Redirecting to site...");
		else if (flags & GTK_MOZ_EMBED_FLAG_TRANSFERRING)
			browser->statusMessage = _("Transferring data from site...");
		else if (flags & GTK_MOZ_EMBED_FLAG_NEGOTIATING)
			browser->statusMessage = _("Waiting for authorization...");
		else if (flags & GTK_MOZ_EMBED_FLAG_START)
			browser->statusMessage = _("Loading site...");
		else if (flags & GTK_MOZ_EMBED_FLAG_STOP)
			browser->statusMessage = _("Done.");
	}

	if (status == GTK_MOZ_EMBED_STATUS_FAILED_DNS)
		browser_show_error(browser,_("Site not found."));
	else if (status == GTK_MOZ_EMBED_STATUS_FAILED_CONNECT)
		browser_show_error(browser,_("Failed to connect to site."));
	else if (status == GTK_MOZ_EMBED_STATUS_FAILED_TIMEOUT)
		browser_show_error(browser,_("Failed due to connection timeout."));
	else if (status == GTK_MOZ_EMBED_STATUS_FAILED_USERCANCELED)
		browser->statusMessage = _("User canceled connecting to site.");

	browser_update_status_bar_text (browser);
}

/**
 * progress_change_cb: GTKMOZEMBED SIGNAL, emitted any time that there is a change in the progress of
 * loading a document.
 */
void progress_change_cb (GtkMozEmbed *embed, gint cur, gint max,
			 GaleonBrowser *browser)
{
#ifdef DEBUG_SIGNALS
	g_print("progress_change_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);

	/* length of the document cannot be determined */
	if (max == -1)
	{
		browser->loadPercent = 0;
		browser->bytesLoaded = cur;
		browser->maxBytesLoaded = 0;
	}
	/* sometimes length of the downloaded document is > than the length of the document */
	else if (cur > max) {
		browser->loadPercent = 100;
		browser->bytesLoaded = cur;
		browser->maxBytesLoaded = max;
	} 
	else {
		browser->bytesLoaded = cur;
		browser->maxBytesLoaded = max;
		browser->loadPercent = (!max) ? 0 : (cur * 100) / max;
	}

	browser_update_status_bar_text (browser);

	/* FIXME:  these need tidying up */
	if (browser->loadPercent == 0) {
		GtkWidget *progress = GTK_WIDGET (gnome_appbar_get_progress (GNOME_APPBAR(browser->appbar1)));
		gfloat val;
		
		if (!(browser->active)) {
			browser->active = TRUE;
			g_timeout_add (100, (GSourceFunc) browser_progress_action, browser);
		}

		gtk_progress_set_activity_mode (GTK_PROGRESS(progress), TRUE);
		val = gtk_progress_get_value (GTK_PROGRESS(progress));
		if (++val > 100) val = 0;
		gtk_progress_set_value (GTK_PROGRESS(progress), val);
	} 
	else 
	{
		GtkWidget *progress = GTK_WIDGET(gnome_appbar_get_progress(GNOME_APPBAR(browser->appbar1)));
		browser->active = FALSE;
		gtk_progress_set_activity_mode (GTK_PROGRESS(progress), FALSE);
		gnome_appbar_set_progress (GNOME_APPBAR(browser->appbar1), browser->loadPercent / 100.0);
	}
}

/**
 * link_message_cb: GTKMOZEMBED SIGNAL, emitted when the link message change
 */
void
link_message_cb (GtkMozEmbed *embed, GaleonBrowser *browser)
{
	char *message;

#ifdef DEBUG_SIGNALS
	g_print("link_message_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);

	/* get the link message */
	message = gtk_moz_embed_get_link_message (embed);
	if (browser->link)
	{
		g_free(browser->link);
		browser->link = NULL;
	}
	browser->link = g_strdup(message);

	/* update browser message */
	if (message && (strlen (message) == 0)) 
	{
		browser_update_temp_message (browser, 0);
		browser->link = NULL;
#ifdef CONTEXT_TIMER_MENU
		/* remove contextual menu timer */
		if (browser->cmenu_timer != NULL) 
		{
			g_timer_destroy (browser->cmenu_timer);
			gtk_timeout_remove(browser->cmenu_timeout_id);
			browser->cmenu_timer = NULL;
		}
#endif

	}	
	else
	{
		browser_update_temp_message (browser, message);
#ifdef CONTEXT_TIMER_MENU
		/* create contextual menu timer */
		browser->cmenu_time = gnome_config_get_float(CONF_GENERAL_AUTOSHOW_TIME);
		browser->cmenu_timer = g_timer_new();
		g_timer_start(browser->cmenu_timer);
		browser->cmenu_timeout_id = gtk_timeout_add(50, (GSourceFunc)cmenu_timeout_callback, 
							    (gpointer) browser);
#endif	
	}

}

/**
 * js_status_cb: GTKMOZEMBED SIGNAL, emitted when the Javascript message status change
 */
void
js_status_cb (GtkMozEmbed *embed, GaleonBrowser *browser)
{
	char *message;

#ifdef DEBUG_SIGNALS
	g_print("js_status_cb\n");
#endif
	g_return_if_fail(browser->WMain!=NULL);

	/* get the javascript message */
	message = gtk_moz_embed_get_js_status (embed);

	/* update the status bar message */
	if (message && (strlen (message) == 0))
	{
		browser_update_temp_message (browser, 0);
	}	
	else
	{
		browser_update_temp_message (browser, message);
	}

}

/**
 * open_uri_cb: GTKMOZEMBED SIGNAL, emitted when the document tries to open a new document
 */
gint open_uri_cb (GtkMozEmbed *embed, const char *uri, GaleonBrowser *browser) {
#ifdef DEBUG_SIGNALS
	g_print("open_uri_cb\n");
#endif
	g_return_val_if_fail(browser->WMain!=NULL,FALSE);

#ifdef CONTEXT_TIMER_MENU	
	/* remove contextual menu timer */
	if (browser->cmenu_timer != NULL) 
	{
		g_timer_destroy (browser->cmenu_timer);
		gtk_timeout_remove(browser->cmenu_timeout_id);
		browser->cmenu_timer = NULL;
	}
#endif	
	/* handle foreign protocols, things like "mailto:" */
	if (handle_foreign_protocols(uri))
		return TRUE;

	/* handle foreign mime types, e.g. RPMS */
	if (handle_foreign_mime_types(uri))
	return TRUE;

	/* we haven't handled it, so let mozilla try to load it */
	return FALSE;
}

#ifdef M18
/**
 * dom_mouse_click_cb: GTKMOZEMBED SIGNAL, emitted when user click on the document
 */
gint dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser)
{
	long type;
	gchar *img, *href;
	gpointer docshell;

#ifdef DEBUG_SIGNALS
	g_print("dom_mouse_click_cb\n");
#endif
	g_return_val_if_fail(browser->WMain!=NULL,FALSE);

	type = mozilla_get_context_menu_type(browser->mozEmbed, dom_event, &img, &docshell, &href);

	switch (mozilla_get_mouse_button(dom_event))
	{
	case 3:
		if ((type & CONTEXT_IMAGE) && (type & CONTEXT_LINK))
		{
			context_show_link_img_menu(browser, img, href);
		}	
		else if (type & CONTEXT_IMAGE)
		{
			context_show_img_menu(browser, img);
		}		
		else if (type & CONTEXT_LINK)
		{
			context_show_link_menu(browser, href);
		}		
		else if (type & CONTEXT_DOCUMENT)
		{
			if (!gnome_config_get_bool("/galeon/Appearance/min_doc_context_checkbutton=FALSE"))
			{
				context_show_doc_menu (browser, docshell);
			}
			else
			{
				context_show_minimal_doc_menu (browser, docshell);
			}
		}	
		else if (type  & CONTEXT_OTHER)
		{
			g_print("clicked other components");
		}	
		return TRUE;
		break;

	case 2:
		if ((type & CONTEXT_LINK) && href)
		{
			browser_create_from_url (href);
			return TRUE;
		}
		break;

	case 1: 
		if ((type & CONTEXT_LINK) && href)
		{
			return handle_foreign_protocols (browser->link);
		}
		break;

	default:
		break;
	}

	return FALSE;
}

gint dom_key_press_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser)
{
	int modifier;
	long key;

#ifdef DEBUG_SIGNALS
	g_print("dom_key_press_cb\n");
#endif
	g_return_val_if_fail(browser->WMain!=NULL,FALSE);

	key = mozilla_get_key_pressed(dom_event,&modifier);

/*	g_print("key =%ld\n",key);*/
/*	g_print("modifier = %d\n", modifier);*/

	if (modifier==0)
	{
		switch(key)
		{
		case DOM_VK_PAGE_UP:
			mozilla_scroll(browser->mozEmbed, 0, -1);
			return TRUE;
			break;
		case DOM_VK_PAGE_DOWN:
			mozilla_scroll(browser->mozEmbed, 0, 1);
			return TRUE;
			break;
		case DOM_VK_UP:
			mozilla_scroll(browser->mozEmbed, -1, 0);
			return TRUE;
			break;
		case DOM_VK_DOWN:
			mozilla_scroll(browser->mozEmbed, 1, 0);
			return TRUE;
			break;
		case DOM_VK_HOME:
			mozilla_scroll_begin (browser->mozEmbed);
			return TRUE;
			break;
		case DOM_VK_END:
			mozilla_scroll_end (browser->mozEmbed);
			return TRUE;
			break;

		default:
			break;
		}
	}
	else if ((modifier & ALT_KEY) && (modifier & CTRL_KEY))
	{
		gboolean b;

		switch(key)
		{
		case DOM_VK_V:
			browser_view_source (browser, NULL);
			return TRUE;
			break;
		case DOM_VK_H:
			browser_show_history (browser);
			return TRUE;
			break;
		case DOM_VK_S:
			b = !GTK_CHECK_MENU_ITEM(browser->view_source_mode)->active;
			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(browser->view_source_mode), b); 
							
			mozilla_view_source (browser->mozEmbed, b);
			return TRUE;
			break;
		default:
			break;
		}
	}
	else if (modifier & KEY_CODE)
	{
		switch(key)
		{
		case DOM_VK_F3:
			browser_show_open_dialog(browser);
			return TRUE;
			break;
		case DOM_VK_F6:
			browser_show_find_dialog(browser, NULL);
			return TRUE;
			break;
		default:
			break;
		}
	}
	else if (modifier & ALT_KEY)
	{
		switch(key)
		{
		case DOM_VK_F:
			gtk_signal_emit_by_name(GTK_OBJECT(
				glade_lookup_widget(browser->WMain, "file1")),
				"activate_item");
			return TRUE;
			break;
		case DOM_VK_E:
			gtk_signal_emit_by_name(GTK_OBJECT(
				glade_lookup_widget(browser->WMain, "edit2")),
				"activate_item");
			return TRUE;
			break;
		case DOM_VK_V:
			gtk_signal_emit_by_name(GTK_OBJECT(
				glade_lookup_widget(browser->WMain, "view1")),
				"activate_item");
			return TRUE;
			break;
		case DOM_VK_O:
			gtk_signal_emit_by_name(GTK_OBJECT(
				glade_lookup_widget(browser->WMain, "bookmarks")),
				"activate_item");
			return TRUE;
			break;
		case DOM_VK_G:
			gtk_signal_emit_by_name(GTK_OBJECT(
				glade_lookup_widget(browser->WMain, "go1")),
				"activate_item");
			return TRUE;
			break;
		case DOM_VK_H:
			gtk_signal_emit_by_name(GTK_OBJECT(
				glade_lookup_widget(browser->WMain, "help1")),
				"activate_item");
			return TRUE;
			break;
		case DOM_VK_N:
			browser_create();
			return TRUE;
			break;
		case DOM_VK_LEFT:
			gtk_moz_embed_go_back(browser->mozEmbed);
			return TRUE;
			break;
		case DOM_VK_RIGHT:
			gtk_moz_embed_go_forward(browser->mozEmbed);
			return TRUE;
			break;
		case DOM_VK_R:
			gtk_moz_embed_reload(browser->mozEmbed, GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
			return TRUE;
			break;
		case DOM_VK_S:
			gtk_moz_embed_stop_load (browser->mozEmbed);
			return TRUE;
			break;
		case DOM_VK_T:
			browser_show_temporary_bookmarks (browser);
			return TRUE;
			break;
		default:
			break;

		}
	}
	else if (modifier & CTRL_KEY)
	{		
		GtkCheckMenuItem *mi;

		switch(key)
		{
		case DOM_VK_A:
			browser_add_temporary_bookmark (browser);
			return TRUE;
			break;
		case DOM_VK_O:
			browser_show_openurl_dialog(browser);
			return TRUE;
			break;		
		case DOM_VK_S:
			browser_show_save_dialog(browser, NULL);
			return TRUE;
			break;		
		case DOM_VK_W:
			browser_close(browser);
			return TRUE;
			break;		
		case DOM_VK_Q:
			galeon_exit();
			return TRUE;
			break;		
		case DOM_VK_T:
			mi = GTK_CHECK_MENU_ITEM(browser->view_toolbar);
			gtk_check_menu_item_set_active(mi, !mi->active);
			return TRUE;
			break;		
		case DOM_VK_R:
			mi = GTK_CHECK_MENU_ITEM(browser->view_statusbar);
			gtk_check_menu_item_set_active(mi, !mi->active);
			return TRUE;
			break;		
		case DOM_VK_F:
			mi = GTK_CHECK_MENU_ITEM(browser->view_fullscreen);
			gtk_check_menu_item_set_active(mi, !mi->active);
			return TRUE;
			break;		
		case DOM_VK_B:
			browser_edit_bookmarks (browser);
			return TRUE;
			break;		
		default:
			break;

		}
	}
	/* seems broken in mozilla */
	else if (modifier & SHIFT_KEY)
	{		
		switch(key)
		{
		default:
			break;

		}
	}

	return FALSE;

}

void 
size_to_cb (GtkMozEmbed *embed, gint width, gint height, GaleonBrowser *browser)
{
	gtk_widget_set_usize (browser->WMain, width, height);
}

gint dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser)
{
	long type;
	gchar *img, *href;
	gpointer docshell;

	g_return_val_if_fail (browser->WMain != NULL, FALSE);

	switch (mozilla_get_mouse_button(dom_event)) {
	case 1:
		type = mozilla_get_context_menu_type /* this should be renamed */
			(browser->mozEmbed, dom_event, &img, &docshell, &href);
		
		if (type & CONTEXT_LINK) {
			g_print ("%s\n", browser->link);
			if (dragging_link) g_free (dragging_link);
			dragging_link = g_strdup (browser->link);
		}

		return FALSE; /* pass the event */
		break;
	case 2: /* drag with other buttons? */
	case 3:
	default:
		return FALSE;
		break;
	}
	return FALSE;
}

gint dom_mouse_over_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser)
{
	g_return_val_if_fail (browser->WMain != NULL, FALSE);
	if ( ! link_drag_types_tl)
		link_drag_types_tl = gtk_target_list_new 
			(link_drag_types, link_drag_types_num_items);

	if (dragging_link) {
		gchar *old_dragging_link;

		old_dragging_link = gtk_object_get_data 
			(GTK_OBJECT (browser->WMain), "dragging_link");
		if (old_dragging_link) g_free (old_dragging_link);
		gtk_object_set_data (GTK_OBJECT (browser->WMain), "dragging_link", 
				     g_strdup (dragging_link));
		g_free (dragging_link);
		dragging_link = NULL;
		gtk_drag_begin (browser->WMain, link_drag_types_tl, 
				GDK_ACTION_COPY | GDK_ACTION_LINK, 1, NULL);
		return TRUE;
	}
	return FALSE;
}

gint dom_mouse_up_cb (GtkMozEmbed *embed, gpointer dom_event, GaleonBrowser *browser)
{

	if (dragging_link) {
		g_free (dragging_link);
		dragging_link = NULL;
	}
	return FALSE;
}

void
link_drag_data_get (GtkWidget *widget, GdkDragContext *context,
		    GtkSelectionData *selection_data, 
		    guint info, guint time, 
		    GaleonBrowser *browser)
{
	gchar *link;
	g_return_if_fail (browser->WMain != NULL);

	link = gtk_object_get_data (GTK_OBJECT (widget), "dragging_link");

	if (link) {
		switch (info) {
		case DND_TARGET_STRING:
		case DND_TARGET_NETSCAPE_URL:
		case DND_TARGET_GALEON_URL:
			gtk_selection_data_set 
				(selection_data, selection_data->target,
				 8, link, strlen (link));
			break;
		default:
			g_warning ("Unknown DND type");
			break;
		}
	} 
}

#endif
