/*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"

/* local types */

typedef struct
{ 
	gint value;
	gchar *name;
	GtkCTreeNode **node_ref;
	gboolean expand;
} preferences_tree_item;

typedef struct
{
	GtkWidget *mime_list;
	GtkWidget *auto_download_optionmenu;
	GtkWidget *ask_download_dir_optionmenu;
	GtkWidget *location_optionmenu;
	GtkWidget *toolbar_optionmenu;
	GtkWidget *startpage_entry;
	GtkWidget *startpage_optionmenu;
	GtkWidget *new_optionmenu;
	GtkWidget *bookmarks_optionmenu;
	GtkWidget *java_optionmenu;
	GtkWidget *javascript_optionmenu;
	GtkWidget *download_dir_entry;
	GtkWidget *error_optionmenu;
	GtkWidget *completion_optionmenu;
	GtkWidget *show_spinner_check;
	GtkWidget *http_proxy_entry;
	GtkWidget *ftp_proxy_entry;
	GtkWidget *http_proxy_port_spin;
	GtkWidget *ftp_proxy_port_spin;
        GtkWidget *no_proxies_for_entry;
	GtkWidget *disk_cache_entry;
	GtkWidget *mem_cache_entry;
	GtkWidget *serif_font_fpicker;
	GtkWidget *sansserif_font_fpicker;
	GtkWidget *monospace_font_fpicker;
	GtkWidget *size_font_spin;
	GtkWidget *history_expire_spin;
	GtkWidget *image_optionmenu;
	GtkWidget *ftp_optionmenu;
	GtkWidget *menu_ctree;
	GtkWidget *show_location_label_check;
	GtkWidget *show_go_button_check;
	GtkWidget *use_external_source_viewer_check;
        GtkWidget *external_source_viewer_entry;
	GtkWidget *crash_optionmenu;
	GtkWidget *newwindows_optionmenu;
	GtkWidget *windows_title_entry;
	GtkWidget *myportal_stylesheet_entry;
	GtkWidget *spinner_dir_entry;
	GtkWidget *autodetect_charset_optionmenu;
	GtkWidget *default_charset_entry;
	GtkWidget *lang_list;
	GtkWidget *background_color;
	GtkWidget *text_color;
	GtkWidget *unvisited_link_color;
	GtkWidget *visited_link_color;
	GtkWidget *use_provided_colors_optionmenu;
	GtkWidget *underline_links_checkbutton;
	GtkWidget *http_version_optionmenu;
	GtkWidget *proxy_keepalive_checkbutton;
	GtkWidget *min_doc_context_checkbutton;
} preferences_widgets;

/* local function prototypes */
static void preferences_load_mime(void);
static void preferences_save_mime(void);
static void preferences_lookup_widgets(GtkWidget *widget,
				       preferences_widgets *pw);
static void build_tree(GtkWidget *ctree, GtkCTreeNode *parent,
		       preferences_tree_item *item);
static void set_config_from_editable(GtkWidget *editable, gchar *config_name);
static void set_config_from_optionmenu(GtkWidget *optionmenu,
				       gchar *config_name);
static void set_config_from_fontpicker(GtkWidget *fontpicker,
				       gchar *config_name);
static void set_config_from_spin_button(GtkWidget *spinbutton,
					gchar *config_name);
static void set_config_from_spin_button_float(GtkWidget *spinbutton,
					gchar *config_name);
static void set_config_from_togglebutton(GtkWidget *togglebutton,
					 gchar *config_name);
static void set_config_from_int_entry(GtkWidget *entry, gchar *config_name);
static void set_config_from_color (GtkWidget *colorpicker, gchar *config_name);
static void set_editable_from_config(GtkWidget *editable, gchar *config_name);
static void set_optionmenu_from_config(GtkWidget *optionmenu,
				       gchar *config_name);
static void set_fontpicker_from_config(GtkWidget *fontpicker,
				       gchar *config_name);
static void set_spin_button_from_config(GtkWidget *spinbutton,
					gchar *config_name);
static void set_spin_button_float_from_config(GtkWidget *spinbutton,
					gchar *config_name);
static void set_int_entry_from_config(GtkWidget *entry, gchar *config_name);
static void set_togglebutton_from_config(GtkWidget *togglebutton,
					 gchar *config_name);
static void set_list_from_config(GtkWidget *list, gchar *config_name);
static void set_config_from_list(GtkWidget *list, gchar *config_name);
static void set_color_from_config (GtkWidget *colorpicker, gchar *config_name);


/**
 * preferences_load: load Galeon preferences needed at startup
 */
void
preferences_load(void)
{
	preferences_load_mime();
}

/**
 * preferences_save: save all Galeon preferences
 */
void
preferences_save(void)
{
	preferences_save_mime();
	gnome_config_sync();
}

/* default MIME types */
struct { gchar *name; gchar *exts; MimeAction action; } default_mimetypes[] =
{
	{ "application/x-bzip", "bz2",      SAVE_TO_DISK },
	{ "audio/mp3",          "mp3",      SAVE_TO_DISK },
	{ "video/mpeg",         "mpeg,mpg", SAVE_TO_DISK },
	{ "application/x-tar",  "tar",      SAVE_TO_DISK },
	{ "application/x-gzip", "gz",       SAVE_TO_DISK },
	{ "application/zip",    "zip",      SAVE_TO_DISK },	
	{ "application/x-rpm",  "rpm",      SAVE_TO_DISK },
	{ NULL, NULL, 0 } /* terminator -- must be last */
};

/**
 * mime_delete_cb: type-correct call back to delete MIME types
 */
static void
mime_delete_cb(gpointer m, gpointer data)
{
	mime_delete((MimeItem *)m);
}

/**
 * preferences_load_mime: load MIME types and details of their handlers
 */
static void
preferences_load_mime(void)
{
	gint i, count;
	gchar *mime_path;
	gchar **mime_type;
	MimeAction action;

	/* free up any existing list of MIME types */
	if (mime_types != NULL)
	{
		g_list_foreach(mime_types, mime_delete_cb, NULL);
		g_list_free(mime_types);
		mime_types = NULL;
	}

	/* working in the galeon/mime configuration hierarchy */
	gnome_config_push_prefix(CONF_MIME);
     
	/* find out the number of configured MIME types */
	count = gnome_config_get_int("count=-1");

	/* if this isn't yet set, copy in the defaults */
	if (count == -1)
	{
		for (i = 0; default_mimetypes[i].name != NULL; i++)
		{
			mime_add(default_mimetypes[i].name, NULL,
				 default_mimetypes[i].exts,
				 default_mimetypes[i].action);
		}
	}

	/* otherwise (count >= 0) read from configuration structure */
	for (i = 0; i < count; i++)
	{
		gint c;

		/* generate the key */
		mime_path = g_strdup_printf("mime%d", i);

		/* lookup the data */
		gnome_config_get_vector(mime_path, &c, &mime_type);
		if (!strcmp(mime_type[2], "SAVE_TO_DISK"))
			action = SAVE_TO_DISK;
		else if (!strcmp(mime_type[2], "RUN_PROGRAM"))
			action = RUN_PROGRAM;
		else if (!strcmp(mime_type[2], "ASK_ACTION"))
			action = ASK_ACTION;
		else
		{
			g_warning("preferences_load_mime: unknown mime type");
			action = SAVE_TO_DISK;
		}

		/* build the mime type */
		mime_add(mime_type[0], mime_type[3], mime_type[1], action);

		/* free allocated strings */
		g_free(mime_path);
		/* FIXME: dont we need to free the vector also...? -- MattA */
	}

	/* ! CONF_MIME */
	gnome_config_pop_prefix();
}

/**
 * preferences_save_mime: save MIME types and handlers 
 */
void
preferences_save_mime(void)
{
	GList *l;
	gchar *mime_type[4];
	gchar *mime_path;
	gint i, count;
  
	/* working in the galeon/mime configuration hierarchy */
	gnome_config_push_prefix(CONF_MIME);

	/* write out the number of MIME types */
	count = g_list_length(mime_types);
	gnome_config_set_int("count",  count);

	/* write out each MIME type entry as string vectors */
	for (l = mime_types, i = 0; l != NULL; l = g_list_next(l), i++)
	{
		/* get the MIME type */
		MimeItem *m = (MimeItem *)(l->data);

		/* fill the entries into the vector */
		mime_type[0] = m->name;
		mime_type[1] = g_strjoinv(",", m->extensions);
		switch (m->action)
		{
		case SAVE_TO_DISK : mime_type[2] = "SAVE_TO_DISK"; break;
		case RUN_PROGRAM  : mime_type[2] = "RUN_PROGRAM";  break;
		case ASK_ACTION   : mime_type[2] = "ASK_ACTION";   break;
		}
		mime_type[3] = m->program;

		/* write the vector out to the config */
		mime_path = g_strdup_printf("mime%d", i);
		gnome_config_set_vector(mime_path, 4,
					(const gchar **)mime_type);

		/* free allocated strings */
		g_free(mime_type[1]);
		g_free(mime_path);
	}

	/* ! CONF_MIME */
	gnome_config_pop_prefix();
}

/**
 * preferences_process: read all the preferences values out of the
 * dialog box widget and store them in the configuration space
 */
void
preferences_process(GtkWidget *widget)
{
        preferences_widgets pw;

	/* lookup all the widgets */
	preferences_lookup_widgets(widget, &pw);

	/* set each of the general configuration options */
	gnome_config_push_prefix("/galeon/General/");
	set_config_from_editable(pw.startpage_entry, "startpage");
	set_config_from_optionmenu(pw.startpage_optionmenu, "startpage_type");
	set_config_from_optionmenu(pw.new_optionmenu, "newpage_type");
	set_config_from_optionmenu(pw.bookmarks_optionmenu, "bookmark_title");
	set_config_from_optionmenu(pw.completion_optionmenu,
				   "completion_type");
	set_config_from_togglebutton(pw.show_spinner_check, "show_spinner");
	set_config_from_optionmenu(pw.newwindows_optionmenu, "newwindows");
	gnome_config_pop_prefix();	
	
	/* process downloading preferences */
	gnome_config_push_prefix("/galeon/Downloading/");
	set_config_from_optionmenu(pw.auto_download_optionmenu,
				   "auto_download");
	set_config_from_optionmenu(pw.ask_download_dir_optionmenu,
				   "ask_for_download_dir");
	set_config_from_editable(pw.download_dir_entry, "download_dir");
	gnome_config_pop_prefix();
	
	/* process appearance preferences */
	gnome_config_push_prefix("/galeon/Appearance/");
	set_config_from_optionmenu(pw.error_optionmenu, "error");
	set_config_from_optionmenu(pw.location_optionmenu, "url_location");
	set_config_from_optionmenu(pw.toolbar_optionmenu, "toolbar_type");
	set_config_from_fontpicker(pw.serif_font_fpicker, "serif_font");
	set_config_from_fontpicker(pw.sansserif_font_fpicker,
				   "sansserif_font");
	set_config_from_fontpicker(pw.monospace_font_fpicker,
				   "monospace_font");
	set_config_from_spin_button(pw.size_font_spin, "size_font");
	set_config_from_togglebutton(pw.show_location_label_check,
				     "show_location_label");
	set_config_from_togglebutton(pw.show_go_button_check,
				     "show_go_button");
	set_config_from_editable(pw.windows_title_entry, "windows_title");
	set_config_from_editable(pw.myportal_stylesheet_entry,
				 "myportal_stylesheet");
	set_config_from_editable(pw.spinner_dir_entry, "spinner_dir");
	set_config_from_color (pw.background_color, "background_color");
	set_config_from_color (pw.text_color, "text_color");
	set_config_from_color (pw.unvisited_link_color, "unvisited_link_color");
	set_config_from_color (pw.visited_link_color, "visited_link_color");
	set_config_from_optionmenu (pw.use_provided_colors_optionmenu, 
				    "use_provided_colors");
	set_config_from_togglebutton (pw.underline_links_checkbutton, 
				      "underline_links");
	set_config_from_togglebutton (pw.min_doc_context_checkbutton, 
				      "min_doc_context_checkbutton");
	gnome_config_pop_prefix();
	
        /* process handlers preferences */
	gnome_config_push_prefix("/galeon/Handlers/");
	set_config_from_optionmenu(pw.ftp_optionmenu, "ftp");
	set_config_from_togglebutton (pw.use_external_source_viewer_check,
				      "use_external_source_viewer");
	set_config_from_editable (pw.external_source_viewer_entry, 
				  "external_source_viewer");
	gnome_config_pop_prefix();
 
        /* process advanced preferences */
	gnome_config_push_prefix("/galeon/Advanced/");
	set_config_from_editable(pw.http_proxy_entry, "http_proxy");
	set_config_from_editable(pw.ftp_proxy_entry, "ftp_proxy");
	set_config_from_editable(pw.no_proxies_for_entry, "no_proxies_for");
	set_config_from_spin_button(pw.http_proxy_port_spin,
				    "http_proxy_port");
	set_config_from_spin_button(pw.ftp_proxy_port_spin, "ftp_proxy_port");
	set_config_from_int_entry(pw.disk_cache_entry, "disk_cache_size");
	set_config_from_int_entry(pw.mem_cache_entry, "mem_cache_size");
	set_config_from_optionmenu(pw.java_optionmenu, "java_enabled");
	set_config_from_optionmenu(pw.javascript_optionmenu,
				   "javascript_enabled");
	set_config_from_spin_button(pw.history_expire_spin, "history_expire");
	set_config_from_optionmenu(pw.image_optionmenu, "image_loading_type");
	set_config_from_optionmenu(pw.crash_optionmenu, "crash_recovery");
	set_config_from_optionmenu(pw.autodetect_charset_optionmenu, "autodetect_charset");
	set_config_from_editable (pw.default_charset_entry, "default_charset");
	set_config_from_list (pw.lang_list, "lang");
	set_config_from_optionmenu(pw.http_version_optionmenu, "http_version"); 
	set_config_from_togglebutton(pw.proxy_keepalive_checkbutton, "proxy_keepalive");
	gnome_config_pop_prefix();

	/* save all these preferences to disk */
	preferences_save();
}

void
preferences_fill(GtkWidget *widget)
{
	preferences_widgets pw;
	GtkCTreeNode *browsing_node;
	GtkCTreeNode *user_interface_node;
	GtkCTreeNode *handlers_node;
	GtkCTreeNode *rendering_node;
	GtkCTreeNode *advanced_node;
  
	preferences_tree_item main_tree[] =
	{
		{  0, _("Browsing"),       &browsing_node,       TRUE  },
		{  0, _("User Interface"), &user_interface_node, TRUE  },
		{  0, _("Handlers"),       &handlers_node,       TRUE  },
		{  0, _("Rendering"),      &rendering_node,      FALSE  },
		{  0, _("Advanced"),       &advanced_node,       FALSE },		
		{ -1, NULL, NULL, FALSE } /* terminator */
	};

	preferences_tree_item browsing_tree[] =
	{
		{  1, _("Homepage"),        NULL, FALSE },
		{  2, _("Bookmarks"),       NULL, FALSE },
		{  3, _("History"),         NULL, FALSE },
		{  4, _("Java"),            NULL, FALSE },
		{  5, _("Filtering"),       NULL, FALSE },
		{ -1, NULL, NULL, FALSE } /* terminator */
        };

	preferences_tree_item user_interface_tree[] =
	{
		{  6, _("Layout"),          NULL, FALSE },
		{  7, _("Spinner"),         NULL, FALSE },
		{  8, _("Context Menus"),      NULL, FALSE },
		{  9, _("Completions"),     NULL, FALSE },
		{ 10, _("Crash Recovery"),  NULL, FALSE },
		{ -1, NULL, NULL, FALSE } /* terminator */
        };

	preferences_tree_item handlers_tree[] =
	{
		{ 11, _("Downloading"),     NULL, FALSE },
		{ 12, _("View Source"),     NULL, FALSE },
/* removed whilst we experiment with other approaches to
   handling mime types, may be removed altogether eventually -- MattA 2/10/00
		{ 13, _("MIME Types"),      NULL, FALSE },
*/
		{ -1, NULL, NULL, FALSE } /* terminator */
	};

	preferences_tree_item rendering_tree[] =
	{
		{ 14, _("Fonts"),           NULL, FALSE },
		{ 15, _("Colors"),          NULL, FALSE },
		{ 18, _("Languages"),       NULL, FALSE },
		{ -1, NULL, NULL, FALSE } /* terminator */
	};

	preferences_tree_item advanced_tree[] =
	{
		{ 16, _("Proxy"),           NULL, FALSE },
		{ 17, _("Cache"),           NULL, FALSE },
		{ 19, _("Network"),         NULL, FALSE },
		{ -1, NULL, NULL } /* terminator */
	};

	/* lookup all the widgets */
	preferences_lookup_widgets(widget, &pw);

	/* build the preferences tree */
	build_tree(pw.menu_ctree, NULL, main_tree);
	build_tree(pw.menu_ctree, browsing_node, browsing_tree);
	build_tree(pw.menu_ctree, user_interface_node, user_interface_tree);
	build_tree(pw.menu_ctree, handlers_node, handlers_tree);
	build_tree(pw.menu_ctree, rendering_node, rendering_tree);
	build_tree(pw.menu_ctree, advanced_node, advanced_tree);

	/* fill general */
	gnome_config_push_prefix("/galeon/General/");
	set_editable_from_config(pw.startpage_entry,
				 "startpage=www.gnome.org");
	set_optionmenu_from_config(pw.startpage_optionmenu,
				   "startpage_type=0");
	set_optionmenu_from_config(pw.new_optionmenu, "newpage_type=0");
	set_optionmenu_from_config(pw.bookmarks_optionmenu, 
				   "bookmark_title=0");
	set_optionmenu_from_config(pw.completion_optionmenu,
				   "completion_type=0");
	set_togglebutton_from_config(pw.show_spinner_check, "show_spinner=1");
	set_optionmenu_from_config(pw.newwindows_optionmenu, "newwindows=0");
	gnome_config_pop_prefix();
	
	/* process downloading preferences */
	gnome_config_push_prefix("/galeon/Downloading/");
	set_optionmenu_from_config(pw.auto_download_optionmenu,
				   "auto_download=0");
	set_optionmenu_from_config(pw.ask_download_dir_optionmenu,
				   "ask_for_download_dir=0");
	set_editable_from_config(pw.download_dir_entry, "download_dir=");
	gnome_config_pop_prefix();
	
	/* process appearance preferences */
	gnome_config_push_prefix("/galeon/Appearance/");
	set_optionmenu_from_config(pw.error_optionmenu, "error=0");
	set_optionmenu_from_config(pw.location_optionmenu, "url_location=1");
	set_optionmenu_from_config(pw.toolbar_optionmenu, "toolbar_type=2");
	set_fontpicker_from_config(pw.serif_font_fpicker,
	    "serif_font=-*-times-medium-r-normal-*-14-*-*-*-*-*-*-*");
	set_fontpicker_from_config(pw.sansserif_font_fpicker,
            "sansserif_font=-*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*");
	set_fontpicker_from_config(pw.monospace_font_fpicker,
	    "monospace_font=-*-courier-medium-r-normal-*-14-*-*-*-*-*-*-*");
	set_spin_button_from_config(pw.size_font_spin, "size_font=14");
	set_togglebutton_from_config(pw.show_location_label_check,
				     "show_location_label=TRUE");
	set_togglebutton_from_config(pw.show_go_button_check,
				     "show_go_button=TRUE");
	set_editable_from_config(pw.windows_title_entry,
				 "windows_title=Galeon - %s");
	set_editable_from_config(pw.myportal_stylesheet_entry,
				 "myportal_stylesheet=");
	set_editable_from_config(pw.spinner_dir_entry,
				 "spinner_dir=" SHARE_DIR);
	set_color_from_config (pw.background_color, "background_color=#FFFFFF");
	set_color_from_config (pw.text_color, "text_color=#000000");
	set_color_from_config (pw.unvisited_link_color, "unvisited_link_color=#0000FF");
	set_color_from_config (pw.visited_link_color, "visited_link_color=#FF0000");
	set_optionmenu_from_config (pw.use_provided_colors_optionmenu,
				    "use_provided_colors=0");
	set_togglebutton_from_config (pw.underline_links_checkbutton, 
				      "underline_links=TRUE");
	set_togglebutton_from_config (pw.min_doc_context_checkbutton, 
				      "min_doc_context_checkbutton=FALSE");
	gnome_config_pop_prefix();
	
        /* process handlers preferences */
	gnome_config_push_prefix("/galeon/Handlers/");
	set_optionmenu_from_config(pw.ftp_optionmenu, "ftp=0");
	set_togglebutton_from_config (pw.use_external_source_viewer_check, 
				      "use_external_source_viewer=FALSE");
	set_editable_from_config (pw.external_source_viewer_entry, 
				  "external_source_viewer=gnome-edit");
	gnome_config_pop_prefix();
 
        /* process advanced preferences */
	gnome_config_push_prefix("/galeon/Advanced/");
	set_editable_from_config(pw.http_proxy_entry, "http_proxy=");
	set_editable_from_config(pw.ftp_proxy_entry, "ftp_proxy=");
	set_editable_from_config(pw.no_proxies_for_entry, "no_proxies_for=");
	set_spin_button_from_config(pw.http_proxy_port_spin,
				    "http_proxy_port=8080");
	set_spin_button_from_config(pw.ftp_proxy_port_spin,
				    "ftp_proxy_port=8080");
	set_optionmenu_from_config(pw.http_version_optionmenu, "http_version=1.1");
	set_togglebutton_from_config(pw.proxy_keepalive_checkbutton, "proxy_keepalive=TRUE");
	/* these defaults seem a bit tight, even for a 
	 * slimline browser such as Galeon! -- MattA */
	set_int_entry_from_config(pw.disk_cache_entry, "disk_cache_size=7680");
	set_int_entry_from_config(pw.mem_cache_entry, "mem_cache_size=1024");
	set_optionmenu_from_config(pw.java_optionmenu, "java_enabled=TRUE");
	set_optionmenu_from_config(pw.javascript_optionmenu,
				   "javascript_enabled=TRUE");
	set_spin_button_from_config(pw.history_expire_spin,
				    "history_expire=100");
	set_optionmenu_from_config(pw.image_optionmenu,
				   "image_loading_type=0");
	set_optionmenu_from_config(pw.crash_optionmenu, "crash_recovery=1");
	set_optionmenu_from_config(pw.autodetect_charset_optionmenu, "autodetect_charset=0");
 	set_editable_from_config (pw.default_charset_entry,"default_charset=Western (ISO-8859-1)"); 
 	set_list_from_config (pw.lang_list,"lang=English\\ (en)"); 
	gnome_config_pop_prefix();

        /* process mime types */
	preferences_mime_update();
}

void
preferences_mime_update(void)
{
	GList *l;
	gchar *text[2] = { NULL, NULL };
	GtkWidget *clist;

	/* FIXME... */
	clist = glade_lookup_widget(GTK_WIDGET(prefs), "mime_list");

	/* freeze and empty the list */
	gtk_clist_freeze (GTK_CLIST(clist));
	gtk_clist_clear (GTK_CLIST(clist));

	/* re-insert every element */
	for (l = mime_types; l != NULL; l = g_list_next(l))
	{
		MimeItem *m = (MimeItem *)(l->data);

		/* insert the row */
		text[0] = g_strdup (m->name);
		m->row = gtk_clist_append (GTK_CLIST(clist), text);
		gtk_clist_set_row_data (GTK_CLIST(clist), m->row, m);
	}

	/* thaw the list -- make changes visible */
	gtk_clist_thaw (GTK_CLIST(clist));	
}

void
set_toolbar_style_global(void)
{
	GSList *l;

	for (l = window_list; l != NULL; l = l->next)
		set_toolbar_style_window((GaleonBrowser *) l->data);
}

void
set_toolbar_style_window(GaleonBrowser *browser)
{
	GtkToolbarStyle style = gnome_config_get_int ("/galeon/Appearance/toolbar_type=2");
	gint url_location = gnome_config_get_int ("/galeon/Appearance/url_location=1");
	gboolean show_location_label = gnome_config_get_bool
		("/galeon/Appearance/show_location_label=TRUE");
	gboolean show_go_button = gnome_config_get_bool
		("/galeon/Appearance/show_go_button=TRUE");

	/* add/remove spinner from toolbar */
	if (browser->spinner!=NULL)
		spinner_destroy(browser);
	if (gnome_config_get_bool("/galeon/General/show_spinner=true"))
		spinner_create(browser);

	/* set main and go toolbar style */
	gtk_toolbar_set_style(GTK_TOOLBAR(browser->main_toolbar), style);
	gtk_toolbar_set_style(GTK_TOOLBAR(browser->go_toolbar1), style);
	/* This toolbar should always be text (to avoid make it too tall) */
	gtk_toolbar_set_style(GTK_TOOLBAR(browser->go_toolbar2), GTK_TOOLBAR_TEXT);

	/* Set toolbar and menubar relief acording to user gnome preferences */
	if (gnome_preferences_get_menubar_relief ()) {
		gnome_dock_item_set_shadow_type
			(GNOME_DOCK_ITEM(browser->menubar->parent),
			 GTK_SHADOW_OUT);
	} else {
		gnome_dock_item_set_shadow_type 
			(GNOME_DOCK_ITEM(browser->menubar->parent), 
			 GTK_SHADOW_NONE);
	}
	if (gnome_preferences_get_toolbar_relief ()) {
		gnome_dock_item_set_shadow_type 
			(GNOME_DOCK_ITEM (browser->main_dockitem), 
			 GTK_SHADOW_OUT);
		gnome_dock_item_set_shadow_type 
			(GNOME_DOCK_ITEM (browser->location_dockitem), 
			 GTK_SHADOW_OUT);
	} else {
		gnome_dock_item_set_shadow_type 
			(GNOME_DOCK_ITEM (browser->main_dockitem), 
			 GTK_SHADOW_NONE);
		gnome_dock_item_set_shadow_type 
			(GNOME_DOCK_ITEM (browser->location_dockitem), 
			 GTK_SHADOW_NONE);
	}

	if (url_location==1) /* button toolbar */
	{
		browser->entry1 = GTK_EDITABLE(gnome_entry_gtk_entry (GNOME_ENTRY(browser->main_toolbar_gnomeentry)));
		browser->eUrl = browser->main_toolbar_gnomeentry;

		/* show button toolbar url entry */
		gtk_widget_show (browser->main_toolbar_gnomeentry);

		/* hide drag and drop icon */
		gtk_widget_show (browser->drag_pixmap);

		/* hide/show location label */
		if (show_location_label)
		{
			gtk_widget_show (browser->location_label1);
		}
		else 
		{
			gtk_widget_hide (browser->location_label1);
		}

		/* hide/show go button */
		if (show_go_button)
		{
			gtk_widget_show (browser->go_toolbar1);
		}
		else 
		{
			gtk_widget_hide (browser->go_toolbar1);
		}

		/* window must exist before we can hide items in it */
		gtk_widget_show (browser->WMain);

		/* hide location toolbar */
		if (GTK_CHECK_MENU_ITEM(browser->view_toolbar)->active)
			gtk_widget_hide (browser->location_dockitem);

		gtk_widget_queue_resize (GTK_WIDGET(browser->mozEmbed));
	}		
	else /* separate toolbar */
	{
		browser->entry1 = GTK_EDITABLE(gnome_entry_gtk_entry (GNOME_ENTRY(browser->location_toolbar_gnomeentry)));
		browser->eUrl = browser->location_toolbar_gnomeentry;

		/* show location toolbar */
		if (GTK_CHECK_MENU_ITEM(browser->view_toolbar)->active)
			gtk_widget_show (browser->location_dockitem);

		/* hide button toolbar location label */
		gtk_widget_hide (browser->location_label1);

		/* hide button toolbar go button */
		gtk_widget_hide (browser->go_toolbar1);

		/* hide button toolbar url entry */
		gtk_widget_hide (browser->main_toolbar_gnomeentry);

		/* hide drag and drop icon */
		gtk_widget_hide (browser->drag_pixmap);

		/* hide/show location label */
		if (show_location_label)
		{
			gtk_widget_show (browser->location_label2);
		}
		else 
		{
			gtk_widget_hide (browser->location_label2);	
		}

		/* hide/show go button */
		if (show_go_button)
		{
			gtk_widget_show (browser->go_toolbar2);
		}
		else
		{
			gtk_widget_hide (browser->go_toolbar2);
		}

		gtk_widget_queue_resize (GTK_WIDGET(browser->mozEmbed));
		gtk_widget_show (browser->WMain);
	}
}


static void
preferences_lookup_widgets(GtkWidget *widget, preferences_widgets *retpw)
{
	/* this "unusual" technique ensures all elements are initialised */
#define Q(name) glade_lookup_widget(widget, name)
	preferences_widgets pw =
	{
	mime_list:                   Q("mime_list"),
	auto_download_optionmenu:    Q("auto_download_optionmenu"),
	ask_download_dir_optionmenu: Q("ask_download_dir_optionmenu"),
	location_optionmenu:         Q("location_optionmenu"),
	toolbar_optionmenu:          Q("toolbar_optionmenu"),
	startpage_entry:             Q("startpage_entry"),
	startpage_optionmenu:        Q("startpage_optionmenu"),
	new_optionmenu:              Q("new_optionmenu"),
	bookmarks_optionmenu:        Q("bookmarks_optionmenu"),
	java_optionmenu:             Q("java_optionmenu"),
	javascript_optionmenu:       Q("javascript_optionmenu"),
	download_dir_entry:          Q("download_dir_entry"),
	error_optionmenu:            Q("error_optionmenu"),
	completion_optionmenu:       Q("completion_optionmenu"),
	show_spinner_check:          Q("show_spinner_checkbutton"),
	http_proxy_entry:            Q("http_proxy_entry"),
	ftp_proxy_entry:             Q("ftp_proxy_entry"),
	no_proxies_for_entry:        Q("no_proxies_for_entry"),
	http_proxy_port_spin:        Q("http_proxy_port_spin"),
	ftp_proxy_port_spin:         Q("ftp_proxy_port_spin"),
	disk_cache_entry:            Q("disk_cache_entry"),
	mem_cache_entry:             Q("mem_cache_entry"),
	serif_font_fpicker:          Q("serif_font_fpicker"),
	sansserif_font_fpicker:      Q("sansserif_font_fpicker"),
	monospace_font_fpicker:      Q("monospace_font_fpicker"),
	size_font_spin:              Q("size_font_spin"),
	history_expire_spin:         Q("history_expire_spin"),
	image_optionmenu:            Q("image_optionmenu"),
	ftp_optionmenu:              Q("ftp_optionmenu"),
	menu_ctree:                  Q("menu_ctree"),
	show_location_label_check:   Q("show_location_label_check"),
	show_go_button_check:        Q("show_go_button_check"),
	use_external_source_viewer_check:
	                             Q("use_external_source_viewer_check"),
	external_source_viewer_entry:Q("external_source_viewer_entry"),
	crash_optionmenu:            Q("crash_optionmenu"),
	newwindows_optionmenu:       Q("newwindows_optionmenu"),
	windows_title_entry:         Q("windows_title_entry"),
	myportal_stylesheet_entry:   Q("myportal_stylesheet_entry"),
	spinner_dir_entry:           Q("spinner_dir_entry"),
	autodetect_charset_optionmenu: Q("autodetect_charset_optionmenu"),
	default_charset_entry:       Q("default_charset_entry"),
	lang_list:                   Q("lang_list"),
	background_color:            Q("background_color"),
	text_color:                  Q("text_color"),
	unvisited_link_color:        Q("unvisited_link_color"),
	visited_link_color:          Q("visited_link_color"),
	use_provided_colors_optionmenu: Q("use_provided_colors_optionmenu"),
	underline_links_checkbutton: Q("underline_links_checkbutton"),
	proxy_keepalive_checkbutton: Q("proxy_keepalive_checkbutton"),
	http_version_optionmenu:     Q("http_version_optionmenu"),
	min_doc_context_checkbutton:     Q("min_doc_context_checkbutton")
	};
#undef Q

	/* copy the structure to the return value */
	*retpw = pw;
}

static void
build_tree(GtkWidget *ctree, GtkCTreeNode *parent, preferences_tree_item *item)
{
	GtkCTreeNode *node;
	gchar *text[] = { NULL, NULL };

	for (; item->name != NULL; item++)
	{
		text[0] = item->name;
		node = gtk_ctree_insert_node(GTK_CTREE(ctree), parent,
					     NULL, text, 0, NULL,
					     NULL, NULL, NULL,
					     FALSE, item->expand);

		gtk_ctree_node_set_row_data(GTK_CTREE(ctree), node,
					    (gpointer)(item->value));

		if (item->value == 0)
		{
			gtk_ctree_node_set_selectable(GTK_CTREE(ctree),
						      GTK_CTREE_NODE(node),
						      FALSE);
		}

		if (item->node_ref != NULL)
		{
			*(item->node_ref) = node;
		}
	}
}

static void
set_config_from_editable(GtkWidget *editable, gchar *config_name)
{
	gchar *value;

	/* get all the text into a new string */
	value = gtk_editable_get_chars (GTK_EDITABLE(editable), 0, -1);

	/* set it in the configuration space */
	gnome_config_set_string (config_name, value);

	/* free the allocated string */
	g_free (value);
}

static void
set_config_from_optionmenu(GtkWidget *optionmenu, gchar *config_name)
{
	GtkWidget *menu;
	GList *list;
	gpointer item;
	gint index;

	/* extract the selection */
	menu = GTK_OPTION_MENU(optionmenu)->menu;
	list = GTK_MENU_SHELL(menu)->children;
	item = gtk_menu_get_active (GTK_MENU(menu));
	index = g_list_index (list, item);
	
	/* set the configuration value */
	gnome_config_set_int (config_name, index);
}

static void
set_config_from_fontpicker(GtkWidget *fontpicker, gchar *config_name)
{
	gchar *font;

	/* get the name of the font */
	font = gnome_font_picker_get_font_name (GNOME_FONT_PICKER(fontpicker));

	/* set the configuration value */
	gnome_config_set_string (config_name, font);
}

static void
set_config_from_spin_button(GtkWidget *spinbutton, gchar *config_name)
{
	gint value;

	/* read the value as an integer */
	value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(spinbutton));

	/* set the configuration value */
	gnome_config_set_int (config_name, value);
}

static void
set_config_from_spin_button_float(GtkWidget *spinbutton, gchar *config_name)
{
	gfloat value;

	/* read the value as an integer */
	value = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(spinbutton));

	/* set the configuration value */
	gnome_config_set_float (config_name, value);
}

static void
set_config_from_togglebutton(GtkWidget *togglebutton, gchar *config_name)
{
	gboolean value;

	/* read the value */
	value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(togglebutton));

	/* set the configuration value */
	gnome_config_set_bool (config_name, value);
}

static void
set_config_from_int_entry(GtkWidget *entry, gchar *config_name)
{
	gchar *value_string;
	gint value;

	/* get the string */
	value_string = gtk_entry_get_text (GTK_ENTRY(entry));

	/* convert it into an integer */
	/* FIXME: atoi is a rubbish function to use, it
	 * doesn't detect any errors. We should fix this -
	 * perhaps by using spin buttons for these things? -- MattA */
	value = atoi(value_string);

	/* set the configuration value */
	gnome_config_set_int (config_name, value);
}

static void
set_config_from_list(GtkWidget *list, gchar *config_name)
{
	GList *children = g_list_copy (GTK_LIST(list)->children);
	guint size = g_list_length (children);
	char **strings = g_malloc (size*sizeof(char*));
	int x = 0;

	while (children) 
	{
		GtkObject       *list_item;
		gchar           *item_data_string;
        
		list_item=GTK_OBJECT(children->data);
		item_data_string=gtk_object_get_data(list_item, "string");
						     
		strings[x] = item_data_string;
			
		children=children->next;
		x++;
	}
	
	/* set the configuration value */
	gnome_config_set_vector (config_name, size, (const char**)strings);

	g_free (children);
	g_free (strings);
}

static void
set_config_from_color (GtkWidget *colorpicker, gchar *config_name)
{
	guint8 r, g, b, a;
	gchar *color_string;
	
	gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (colorpicker), &r, &g, &b, &a);
	color_string = g_strdup_printf ("#%02X%02X%02X", r, g, b);

	/* set the configuration value */
	gnome_config_set_string (config_name, color_string);
}

static void
set_editable_from_config(GtkWidget *editable, gchar *config_name)
{
	gchar *value;

	/* get the current value from the configuration space */
	value = gnome_config_get_string (config_name);

	/* set this string value in the widget */
	gtk_entry_set_text(GTK_ENTRY(editable), value);

	/* free the allocated string */
	g_free (value);
}

static void
set_optionmenu_from_config(GtkWidget *optionmenu, gchar *config_name)
{
	gint index;

	/* get the current value from the configuration space */
	index = gnome_config_get_int (config_name);

	/* set this option value in the widget */
	gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu), index);
}

static void
set_fontpicker_from_config(GtkWidget *fontpicker, gchar *config_name)
{
	gchar *font;

	/* get the current value from the configuration space */
	font = gnome_config_get_string (config_name);

	/* get the name of the font */
	gnome_font_picker_set_font_name (GNOME_FONT_PICKER(fontpicker), font);

	/* free the allocated string */
	g_free (font);
}

static void
set_spin_button_from_config(GtkWidget *spinbutton, gchar *config_name)
{
	gint value;

	/* get the current value from the configuration space */
	value = gnome_config_get_int (config_name);

	/* set this option value in the widget */
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), value);
}

static void
set_spin_button_float_from_config(GtkWidget *spinbutton, gchar *config_name)
{
	gfloat value;

	/* get the current value from the configuration space */
	value = gnome_config_get_float (config_name);

	/* set this option value in the widget */
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), value);
}


static void
set_togglebutton_from_config(GtkWidget *togglebutton, gchar *config_name)
{
	gboolean value;

	/* get the current value from the configuration space */
	value = gnome_config_get_bool (config_name);

	/* set this option value in the widget */
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (togglebutton), value);
}

static void
set_int_entry_from_config(GtkWidget *entry, gchar *config_name)
{
	gchar *value_string;
	gint value;

	/* get the current value from the configuration space */
	value = gnome_config_get_int (config_name);

	/* convert to a string */
	value_string = g_strdup_printf("%d", value);

	/* set this string value in the widget */
	gtk_entry_set_text(GTK_ENTRY(entry), value_string);

	/* free the allocated string */
	g_free (value_string);
}

static void
set_list_from_config(GtkWidget *list, gchar *config_name)
{
	int size;
	gchar **strings;
	GList *items = NULL;
	int k;

	gnome_config_get_vector (config_name, &size, &strings);
	
	if (!strings) return;
	if (!strcmp(strings[0],"")) return;

	for (k=0; k<size ; k++)
	{	
		GtkWidget *list_item;
		char *label;
		
		label = strdup(strings[k]);	
		list_item = gtk_list_item_new_with_label(label);
		gtk_object_set_data(GTK_OBJECT(list_item), "string", label);
		items = g_list_append (items, list_item);
		gtk_widget_show(list_item);
	}

	if (items)
		gtk_list_append_items(GTK_LIST(list), items);
}

static void
set_color_from_config (GtkWidget *colorpicker, gchar *config_name)
{
	guint ir, ig, ib;
	guint8 r, g, b;
	gchar *color_string = gnome_config_get_string (config_name);
	
	sscanf (color_string, "#%2X%2X%2X", &ir, &ig, &ib);
	r = ir;
	g = ig;
	b = ib;
	gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (colorpicker), r, g, b, 0);

	/* set the configuration value */
	gnome_config_set_string (config_name, color_string);
}
